import subprocess
import sys
from pathlib import Path
from unittest import mock

import pytest

from briefcase.commands.base import BaseCommand
from briefcase.console import Console, Log, LogLevel
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.macOS import SigningIdentity, macOSSigningMixin
from briefcase.platforms.macOS.app import macOSAppMixin
from tests.utils import DummyConsole


class DummySigningCommand(macOSAppMixin, macOSSigningMixin, BaseCommand):
    """A dummy command to expose code signing capabilities."""

    command = "sign"

    def __init__(self, base_path, **kwargs):
        kwargs.setdefault("logger", Log())
        kwargs.setdefault("console", Console())
        super().__init__(base_path=base_path / "base_path", **kwargs)
        self.tools.input = DummyConsole()


@pytest.fixture
def dummy_command(tmp_path):
    cmd = DummySigningCommand(base_path=tmp_path)

    # Mock the options object
    cmd.options = mock.MagicMock()
    cmd.options.device = None

    # Mock get_identities
    mock_get_identities = mock.MagicMock()
    cmd.get_identities = mock_get_identities

    cmd.tools.subprocess = mock.MagicMock(spec_set=Subprocess)

    return cmd


def sign_call(
    tmp_path,
    filepath,
    identity,
    entitlements=True,
    runtime=True,
):
    """A test utility method to quickly construct a subprocess call to invoke codesign
    on a file."""
    args = [
        "codesign",
        filepath,
        "--sign",
        identity.id,
        "--force",
    ]
    if entitlements:
        args.extend(
            [
                "--entitlements",
                tmp_path / "base_path/build/first-app/macos/app/Entitlements.plist",
            ]
        )
    if runtime:
        args.extend(
            [
                "--options",
                "runtime",
            ]
        )

    return mock.call(args, stderr=subprocess.PIPE, check=True)


def mock_codesign(results):
    """A utility method for generating codesign side effects.

    :param results: A single error string; or a list of error strings to be returned
        on successive calls. If `None` is included in the list of results, no
        error will be raised for that invocation.
    """

    def _codesign(args, **kwargs):
        if isinstance(results, list):
            result = results.pop(0)
        else:
            result = results

        if result:
            raise subprocess.CalledProcessError(
                returncode=1, cmd=args, stderr=f"{args[1]}: {result}"
            )

    return _codesign


def test_explicit_app_identity_checksum(dummy_command):
    """If the user nominates an app identity by checksum, it is used."""
    # get_identities will return some options.
    dummy_command.get_identities.return_value = {
        "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
        "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
    }

    # The identity will be the one the user specified as an option.
    result = dummy_command.select_identity("11E77FB58F13F6108B38110D5D92233C58ED38C5")

    assert result == SigningIdentity(
        id="11E77FB58F13F6108B38110D5D92233C58ED38C5",
        name="iPhone Developer: Jane Smith (BXAH5H869S)",
    )

    # User input was not solicited
    assert dummy_command.input.prompts == []


def test_explicit_app_identity_name(dummy_command):
    """If the user nominates an app identity by name, it is used."""
    # get_identities will return some options.
    dummy_command.get_identities.return_value = {
        "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
        "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
    }

    # The identity will be the one the user specified as an option.
    result = dummy_command.select_identity("iPhone Developer: Jane Smith (BXAH5H869S)")

    assert result == SigningIdentity(
        id="11E77FB58F13F6108B38110D5D92233C58ED38C5",
        name="iPhone Developer: Jane Smith (BXAH5H869S)",
    )

    # User input was not solicited
    assert dummy_command.input.prompts == []


def test_invalid_app_identity_name(dummy_command):
    """If the user nominates an app identity by name, it is used."""
    # get_identities will return some options.
    dummy_command.get_identities.return_value = {
        "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
        "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
    }

    # The identity will be the one the user specified as an option.
    with pytest.raises(BriefcaseCommandError):
        dummy_command.select_identity("not-an-identity")

    # User input was not solicited
    assert dummy_command.input.prompts == []


def test_implied_app_identity(dummy_command):
    """If there is only one app identity, it will still prompt with ad-hoc as a second
    option."""
    # get_identities will return some options.
    dummy_command.get_identities.return_value = {
        "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
    }

    # Return option 2
    dummy_command.input.values = ["2"]

    result = dummy_command.select_identity()

    assert result == SigningIdentity(
        id="11E77FB58F13F6108B38110D5D92233C58ED38C5",
        name="iPhone Developer: Jane Smith (BXAH5H869S)",
    )

    # User input was solicited
    assert dummy_command.input.prompts


def test_no_app_identities(dummy_command):
    """If there are no identities the user will be prompted to select with only ad-hoc
    as an option."""
    # get_identities will return some options.
    dummy_command.get_identities.return_value = {}

    # Return option 1
    dummy_command.input.values = ["1"]

    result = dummy_command.select_identity()

    # Result is the adhoc identity
    assert result == SigningIdentity()

    # User input was solicited
    assert dummy_command.input.prompts


def test_select_app_identity(dummy_command):
    """The user can select from a list of app identities."""
    # get_identities will return some options.
    dummy_command.get_identities.return_value = {
        "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
        "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
    }

    # Return option 3
    dummy_command.input.values = ["3"]

    result = dummy_command.select_identity()

    # The identity will be second of the returned values
    assert result == SigningIdentity(
        id="11E77FB58F13F6108B38110D5D92233C58ED38C5",
        name="iPhone Developer: Jane Smith (BXAH5H869S)",
    )

    # User input was solicited once
    assert dummy_command.input.prompts == ["> "]


def test_select_installer_identity(dummy_command):
    """The user can select from a list of installer identities."""
    # get_identities is invoked twice - once with app identities, and once with all identities.
    dummy_command.get_identities.side_effect = [
        {
            "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
            "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
        },
        {
            "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
            "4C1067833814CE4738EBD6F8903EC63C238B0CA3": "Developer ID Installer: Example Corp Ltd (Z2K4383DLE)",
            "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
            "8903EC63C238B04C138EBD6F067833814CE47CA3": "Developer ID Installer: Example Corp Ltd (Z2K4383DLE)",
        },
    ]

    # Return option 2
    dummy_command.input.values = ["2"]

    result = dummy_command.select_identity(
        app_identity=SigningIdentity(
            id="38EBD6F8903EC63C238B04C1067833814CE47CA3",
            name="Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
        )
    )

    # The identity will be the second of the returned options; ad-hoc won't be added
    assert result == SigningIdentity(
        id="8903EC63C238B04C138EBD6F067833814CE47CA3",
        name="Developer ID Installer: Example Corp Ltd (Z2K4383DLE)",
    )

    # User input was solicited once
    assert dummy_command.input.prompts == ["> "]


def test_installer_identity_matching_app(dummy_command):
    """The list of possible installer identities includes non-app identities from the
    same team."""
    # get_identities is invoked twice - once with app identities, and once with all identities.
    dummy_command.get_identities.side_effect = [
        {
            "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
            "EBD6F8903EC63C238B0384C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (83DLEZ2K43)",
            "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
        },
        {
            # The app identity that will be selected
            "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
            # A different app identity
            "EBD6F8903EC63C238B0384C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (83DLEZ2K43)",
            # An installer identity that doesn't match the selected app identity
            "1067833814CE4738EB4CD6F8903EC63C238B0CA3": "Developer ID Installer: Example Corp Ltd (83DLEZ2K43)",
            # An installer identity that *does* match the selected app identity
            "4C1067833814CE4738EBD6F8903EC63C238B0CA3": "Developer ID Installer: Example Corp Ltd (Z2K4383DLE)",
            # A different app identity
            "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
            # Another installer identity that match the selected app identity
            "8903EC63C238B04C138EBD6F067833814CE47CA3": "Developer ID Installer: Example Corp Ltd (Z2K4383DLE)",
        },
    ]

    # Return option 2
    dummy_command.input.values = ["2"]

    result = dummy_command.select_identity(
        app_identity=SigningIdentity(
            id="38EBD6F8903EC63C238B04C1067833814CE47CA3",
            name="Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
        )
    )

    # The identity will be the second of the returned options; ad-hoc won't be added,
    # and the installer identity for the other app identity won't be included
    assert result == SigningIdentity(
        id="8903EC63C238B04C138EBD6F067833814CE47CA3",
        name="Developer ID Installer: Example Corp Ltd (Z2K4383DLE)",
    )

    # User input was solicited once
    assert dummy_command.input.prompts == ["> "]


def test_installer_identity_no_match(dummy_command):
    """The list of possible installer identities includes non-app identities from the
    same team."""
    # get_identities is invoked twice - once with app identities, and once with all identities.
    dummy_command.get_identities.side_effect = [
        {
            "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
            "EBD6F8903EC63C238B0384C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (83DLEZ2K43)",
            "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
        },
        {
            # The app identity that will be selected
            "38EBD6F8903EC63C238B04C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
            # A different app identity
            "EBD6F8903EC63C238B0384C1067833814CE47CA3": "Developer ID Application: Example Corp Ltd (83DLEZ2K43)",
            # An installer identity that doesn't match the selected app identity
            "1067833814CE4738EB4CD6F8903EC63C238B0CA3": "Developer ID Installer: Example Corp Ltd (83DLEZ2K43)",
            # A different app identity
            "11E77FB58F13F6108B38110D5D92233C58ED38C5": "iPhone Developer: Jane Smith (BXAH5H869S)",
        },
    ]

    # As there are no viable installer certificates, an error is raised.
    with pytest.raises(
        BriefcaseCommandError,
        match=r"No installer signing identities for team Z2K4383DLE could be found.",
    ):
        dummy_command.select_identity(
            app_identity=SigningIdentity(
                id="38EBD6F8903EC63C238B04C1067833814CE47CA3",
                name="Developer ID Application: Example Corp Ltd (Z2K4383DLE)",
            )
        )

    # User input was never solicited once
    assert dummy_command.input.prompts == []


@pytest.mark.parametrize("verbose", [True, False])
def test_sign_file_adhoc_identity(
    dummy_command,
    adhoc_identity,
    verbose,
    tmp_path,
    capsys,
):
    """If an ad-hoc identity is used, the runtime option isn't used."""
    if verbose:
        dummy_command.logger.verbosity = LogLevel.VERBOSE

    # Sign the file with an ad-hoc identity
    dummy_command.sign_file(tmp_path / "base_path/random.file", identity=adhoc_identity)

    # An attempt to codesign was made without the runtime option
    dummy_command.tools.subprocess.run.assert_has_calls(
        [
            sign_call(
                tmp_path,
                tmp_path / "base_path/random.file",
                identity=adhoc_identity,
                entitlements=False,
                runtime=False,
            ),
        ],
        any_order=False,
    )

    # No console output
    output = capsys.readouterr().out
    assert len(output.strip("\n").split("\n")) == 1


@pytest.mark.parametrize("verbose", [True, False])
def test_sign_file_entitlements(
    dummy_command,
    sekrit_identity,
    verbose,
    tmp_path,
    capsys,
):
    """Entitlements can be included in a signing call."""
    if verbose:
        dummy_command.logger.verbosity = LogLevel.VERBOSE

    # Sign the file with an ad-hoc identity
    dummy_command.sign_file(
        tmp_path / "base_path/random.file",
        identity=sekrit_identity,
        entitlements=tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "Entitlements.plist",
    )

    # An attempt to codesign was made without the runtime option
    dummy_command.tools.subprocess.run.assert_has_calls(
        [
            sign_call(
                tmp_path,
                tmp_path / "base_path/random.file",
                identity=sekrit_identity,
            ),
        ],
        any_order=False,
    )

    # No console output
    output = capsys.readouterr().out
    assert len(output.strip("\n").split("\n")) == 1


@pytest.mark.parametrize("verbose", [True, False])
def test_sign_file_unsupported_format(
    dummy_command,
    sekrit_identity,
    verbose,
    tmp_path,
    capsys,
):
    """If codesign reports an unsupported format, the signing attempt is ignored with a
    warning."""
    if verbose:
        dummy_command.logger.verbosity = LogLevel.VERBOSE

    # FIXME: I'm not sure how to manufacture this in practice.
    dummy_command.tools.subprocess.run.side_effect = mock_codesign(
        "unsupported format for signature"
    )

    # Sign the file
    dummy_command.sign_file(
        tmp_path / "base_path/random.file",
        identity=sekrit_identity,
    )

    # An attempt to codesign was made
    dummy_command.tools.subprocess.run.assert_has_calls(
        [
            sign_call(
                tmp_path,
                tmp_path / "base_path/random.file",
                identity=sekrit_identity,
                entitlements=False,
            ),
        ],
        any_order=False,
    )

    output = capsys.readouterr().out
    if verbose:
        # The console includes a warning about not needing a signature.
        assert "... random.file does not require a signature\n" in output

    # Output only happens if in debug mode
    assert len(output.strip("\n").split("\n")) == (2 if verbose else 1)


@pytest.mark.parametrize("verbose", [True, False])
def test_sign_file_unknown_bundle_format(
    dummy_command,
    sekrit_identity,
    verbose,
    tmp_path,
    capsys,
):
    """If a folder happens to have a .framework extension, the signing attempt is
    ignored with a warning."""
    if verbose:
        dummy_command.logger.verbosity = LogLevel.VERBOSE

    # Raise an error caused by an unknown bundle format during codesign
    dummy_command.tools.subprocess.run.side_effect = mock_codesign(
        "bundle format unrecognized, invalid, or unsuitable"
    )

    # Sign the file
    dummy_command.sign_file(
        tmp_path / "base_path/random.file",
        identity=sekrit_identity,
    )

    # An attempt to codesign was made
    dummy_command.tools.subprocess.run.assert_has_calls(
        [
            sign_call(
                tmp_path,
                tmp_path / "base_path/random.file",
                identity=sekrit_identity,
                entitlements=False,
            ),
        ],
        any_order=False,
    )

    output = capsys.readouterr().out
    if verbose:
        # The console includes a warning about not needing a signature.
        assert "... random.file does not require a signature\n" in output

    # Output only happens if in debug mode
    assert len(output.strip("\n").split("\n")) == (2 if verbose else 1)


@pytest.mark.parametrize("verbose", [True, False])
def test_sign_file_unknown_error(
    dummy_command,
    sekrit_identity,
    verbose,
    tmp_path,
    capsys,
):
    """Any other codesigning error raises an error."""
    if verbose:
        dummy_command.logger.verbosity = LogLevel.VERBOSE

    # Raise an unknown error during codesign
    dummy_command.tools.subprocess.run.side_effect = mock_codesign("Unknown error")

    with pytest.raises(BriefcaseCommandError, match="Unable to code sign "):
        dummy_command.sign_file(
            tmp_path / "base_path/random.file",
            identity=sekrit_identity,
        )

    # An attempt to codesign was made
    dummy_command.tools.subprocess.run.assert_has_calls(
        [
            sign_call(
                tmp_path,
                tmp_path / "base_path/random.file",
                identity=sekrit_identity,
                entitlements=False,
            ),
        ],
        any_order=False,
    )

    # No console output
    output = capsys.readouterr().out
    assert len(output.strip("\n").split("\n")) == 1


@pytest.mark.parametrize("verbose", [True, False])
def test_sign_app(
    dummy_command,
    sekrit_identity,
    first_app_with_binaries,
    verbose,
    tmp_path,
    capsys,
):
    """An app bundle can be signed."""
    if verbose:
        dummy_command.logger.verbosity = LogLevel.VERBOSE

    # Sign the app
    dummy_command.sign_app(
        first_app_with_binaries,
        identity=sekrit_identity,
    )

    # A request has been made to sign all the so and dylib files
    # This acts as a test of the discovery process:
    # * It discovers frameworks
    # * It discovers apps
    # * It discovers Mach-O binaries in various forms and guises
    # * It *doesn't* discover directories
    # * It *doesn't* discover non-Mach-O binaries
    # * It traverses in "depth first" order
    app_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "First App.app"
    )
    lib_path = app_path / "Contents/Resources/app_packages"
    frameworks_path = app_path / "Contents/Frameworks"
    dummy_command.tools.subprocess.run.assert_has_calls(
        [
            sign_call(
                tmp_path,
                lib_path / "subfolder/second_so.so",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                lib_path / "subfolder/second_dylib.dylib",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                lib_path / "special.binary",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                lib_path / "other_binary",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                lib_path / "first_so.so",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                lib_path / "first_dylib.dylib",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                lib_path / "Extras.app/Contents/MacOS/Extras",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                lib_path / "Extras.app",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                frameworks_path / "Extras.framework/Resources/extras.dylib",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                frameworks_path / "Extras.framework",
                identity=sekrit_identity,
            ),
            sign_call(
                tmp_path,
                app_path,
                identity=sekrit_identity,
            ),
        ],
        any_order=True,
    )

    # Also check that files are not signed after their parent directory has been
    # signed. Reduce the files mentions in the calls to the dummy command
    # to a list of path objects, then ensure that the call to sign any given file
    # does not occur *after* it's parent directory.
    sign_targets = [
        Path(call.args[0][1]) for call in dummy_command.tools.subprocess.run.mock_calls
    ]

    parents = set()
    for path in sign_targets:
        # Check parent of path is not in parents
        assert path.parent not in parents
        parents.add(path)

    # Output only happens if in debug mode.
    output = capsys.readouterr().out
    if sys.platform == "win32":
        # In practice, we won't ever actually run signing on win32; but to ensure test
        # coverage we need to. However, win32 doesn't handle executable permissions
        # the same as linux/unix, `unknown.binary` is identified as a signing target.
        # We ignore this discrepancy for testing purposes.
        assert len(output.strip("\n").split("\n")) == (12 if verbose else 1)
    else:
        assert len(output.strip("\n").split("\n")) == (11 if verbose else 1)


@pytest.mark.parametrize("verbose", [True, False])
def test_sign_app_with_failure(
    dummy_command,
    sekrit_identity,
    first_app_with_binaries,
    verbose,
    capsys,
):
    """If signing a single file in the app fails, the error is surfaced."""
    if verbose:
        dummy_command.logger.verbosity = LogLevel.VERBOSE

    # Sign the app. Signing first_dylib.dylib will fail.
    def _codesign(args, **kwargs):
        if Path(args[1]).name == "first_dylib.dylib":
            raise subprocess.CalledProcessError(
                returncode=1, cmd=args, stderr=f"{args[1]}: Unknown error"
            )

    dummy_command.tools.subprocess.run.side_effect = _codesign

    # The invocation will raise an error; however, we can't predict exactly which
    # file will raise an error.
    with pytest.raises(
        BriefcaseCommandError, match=r"Unable to code sign .*first_dylib\.dylib"
    ):
        dummy_command.sign_app(
            first_app_with_binaries,
            identity=sekrit_identity,
        )

    # There has been at least 1 call to sign files. We can't know how many are
    # actually signed, as threads are involved.
    dummy_command.tools.subprocess.run.call_count > 0

    # Output only happens if in debug mode.
    output = capsys.readouterr().out
    if sys.platform == "win32":
        # In practice, we won't ever actually run signing on win32; but to ensure test
        # coverage we need to. However, win32 doesn't handle executable permissions
        # the same as linux/unix, `unknown.binary` is identified as a signing target.
        # We ignore this discrepancy for testing purposes.
        assert len(output.strip("\n").split("\n")) == (7 if verbose else 1)
    else:
        assert len(output.strip("\n").split("\n")) == (6 if verbose else 1)



from collections import namedtuple

import pytest

from briefcase.console import Log
from briefcase.integrations.subprocess import get_process_id_by_command

Process = namedtuple("Process", "info")
process_list_one_proc = [
    Process(
        info=dict(cmdline=["/bin/cmd.sh", "--input", "data"], create_time=20, pid=100)
    )
]

process_list_two_procs_diff_cmd = [
    Process(
        info=dict(
            cmdline=["/bin/first_cmd.sh", "--input", "data"], create_time=20, pid=100
        )
    ),
    Process(
        info=dict(
            cmdline=["/bin/second_cmd.sh", "--input", "data"], create_time=10, pid=200
        )
    ),
]

process_list_two_procs_same_cmd = [
    Process(
        info=dict(cmdline=["/bin/cmd.sh", "--input", "data"], create_time=20, pid=100)
    ),
    Process(
        info=dict(cmdline=["/bin/cmd.sh", "--input", "data"], create_time=10, pid=200)
    ),
]


@pytest.mark.parametrize(
    ("process_list", "command_list", "expected_pid", "expected_stdout"),
    [
        ([], ["/bin/cmd.sh", "--input", "data"], None, ""),
        (process_list_one_proc, ["/bin/cmd.sh", "--input", "data"], 100, ""),
        (process_list_one_proc, ["/bin/random_cmd.sh", "--input", "data"], None, ""),
        (
            process_list_two_procs_diff_cmd,
            ["/bin/first_cmd.sh", "--input", "data"],
            100,
            "",
        ),
        (
            process_list_two_procs_diff_cmd,
            ["/bin/random_cmd.sh", "--input", "data"],
            None,
            "",
        ),
        (
            process_list_two_procs_same_cmd,
            ["/bin/cmd.sh", "--input", "data"],
            100,
            "Multiple running instances of app found. "
            "Using most recently created app process 100.\n",
        ),
        (
            process_list_two_procs_same_cmd,
            ["/bin/random_cmd.sh", "--input", "data"],
            None,
            "",
        ),
    ],
)
def test_get_process_id_by_command_w_command_line(
    process_list,
    command_list,
    expected_pid,
    expected_stdout,
    monkeypatch,
    capsys,
):
    """Finds correct process for command line or returns None."""
    monkeypatch.setattr("psutil.process_iter", lambda attrs: process_list)
    found_pid = get_process_id_by_command(command_list=command_list, logger=Log())
    assert found_pid == expected_pid
    assert capsys.readouterr().out == expected_stdout


@pytest.mark.parametrize(
    ("process_list", "command", "expected_pid", "expected_stdout"),
    [
        ([], "/bin/cmd.sh", None, ""),
        (process_list_one_proc, "/bin/cmd", 100, ""),
        (process_list_one_proc, "/bin/cmd.sh --input data", None, ""),
        (process_list_one_proc, "/bin/cmd.sh", 100, ""),
        (process_list_one_proc, "/bin/random_cmd.sh", None, ""),
        (process_list_two_procs_diff_cmd, "/bin/first_cmd.sh", 100, ""),
        (process_list_two_procs_diff_cmd, "/bin/random_cmd.sh", None, ""),
        (
            process_list_two_procs_same_cmd,
            "/bin/cmd.sh",
            100,
            "Multiple running instances of app found. "
            "Using most recently created app process 100.\n",
        ),
        (process_list_two_procs_same_cmd, "/bin/random_cmd.sh", None, ""),
    ],
)
def test_get_process_id_by_command_w_command(
    process_list,
    command,
    expected_pid,
    expected_stdout,
    monkeypatch,
    capsys,
):
    """Finds correct process for command or returns None."""
    monkeypatch.setattr("psutil.process_iter", lambda attrs: process_list)
    found_pid = get_process_id_by_command(command=command, logger=Log())
    assert found_pid == expected_pid
    assert capsys.readouterr().out == expected_stdout


def test_get_process_id_no_logging(monkeypatch, capsys):
    """If no logger is provided, warnings about ambiguous matches aren't printed."""
    monkeypatch.setattr(
        "psutil.process_iter",
        lambda attrs: process_list_two_procs_same_cmd,
    )
    found_pid = get_process_id_by_command(
        command_list=["/bin/cmd.sh", "--input", "data"]
    )
    assert found_pid == 100
    assert capsys.readouterr().out == ""



import os
import subprocess
from unittest import mock

import pytest

from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.xcode import Xcode


@pytest.fixture
def default_xcode_install_path(tmp_path):
    return tmp_path / "Applications/Xcode.app"


@pytest.fixture
def xcode(default_xcode_install_path):
    """Create a dummy location for Xcode."""
    default_xcode_install_path.mkdir(parents=True, exist_ok=True)
    return os.fsdecode(default_xcode_install_path)


def test_not_installed(tmp_path, mock_tools):
    """If No Xcode is installed, raise an error."""
    mock_tools.subprocess.check_output.side_effect = subprocess.CalledProcessError(
        cmd=["xcode-select", "-p"],
        returncode=2,
    )

    # Test a location where Xcode *won't* be installed
    with pytest.raises(BriefcaseCommandError):
        Xcode.ensure_xcode_is_installed(mock_tools)

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [mock.call(["xcode-select", "-p"])],
        any_order=False,
    )


def test_custom_install_location(default_xcode_install_path, tmp_path, mock_tools):
    """If Xcode is in a non-default location, that's fine."""
    # Create a custom Xcode location
    custom_xcode_location = tmp_path / "custom/Xcode.app"
    custom_xcode_location.mkdir(parents=True, exist_ok=True)

    mock_tools.subprocess.check_output.side_effect = [
        os.fsdecode(custom_xcode_location) + "\n",  # xcode-select -p
        "Xcode 13.3.1\nBuild version 11B500\n",  # xcodebuild -version
    ]

    Xcode.ensure_xcode_is_installed(
        mock_tools,
        xcode_location=default_xcode_install_path,
    )

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )


def test_command_line_tools_only(default_xcode_install_path, mock_tools):
    """If the cmdline tools are installed, but Xcode isn't, raise an error."""
    mock_tools.subprocess.check_output.side_effect = [
        "/Library/Developer/CommandLineTools\n",  # xcode-select -p
        subprocess.CalledProcessError(
            cmd=["xcodebuild", "-version"],
            returncode=1,
            output="xcode-select: error: tool 'xcodebuild' requires Xcode, but "
            "active developer directory '/Library/Developer/CommandLineTools' "
            "is a command line tools instance\n",
        ),
    ]

    with pytest.raises(
        BriefcaseCommandError,
        match=r"You have the Xcode command line tools installed",
    ):
        Xcode.ensure_xcode_is_installed(
            mock_tools,
            xcode_location=default_xcode_install_path,
        )

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )


def test_installed_but_command_line_tools_selected(
    default_xcode_install_path,
    xcode,
    mock_tools,
):
    """If Xcode is installed, but the cmdline tools are selected raise an error."""
    mock_tools.subprocess.check_output.side_effect = [
        xcode + "\n",  # xcode-select -p
        subprocess.CalledProcessError(
            cmd=["xcodebuild", "-version"],
            returncode=1,
            output="xcode-select: error: tool 'xcodebuild' requires Xcode, but "
            "active developer directory '/Library/Developer/CommandLineTools' "
            "is a command line tools instance\n",
        ),
    ]

    with pytest.raises(
        BriefcaseCommandError,
        match=r"Xcode appears to be installed, but the active developer directory ",
    ):
        Xcode.ensure_xcode_is_installed(
            mock_tools,
            xcode_location=default_xcode_install_path,
        )

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )


def test_custom_install_with_command_line_tools(
    default_xcode_install_path,
    tmp_path,
    mock_tools,
):
    """If the cmdline tools are installed, and Xcode is in a non-default location, raise
    an error."""
    # Create a custom Xcode location
    custom_xcode_location = tmp_path / "custom/Xcode.app"
    custom_xcode_location.mkdir(parents=True, exist_ok=True)

    mock_tools.subprocess.check_output.side_effect = [
        "/Library/Developer/CommandLineTools\n",  # xcode-select -p
        subprocess.CalledProcessError(
            cmd=["xcodebuild", "-version"],
            returncode=1,
            output="xcode-select: error: tool 'xcodebuild' requires Xcode, but "
            "active developer directory '/Library/Developer/CommandLineTools' "
            "is a command line tools instance\n",
        ),
    ]

    with pytest.raises(
        BriefcaseCommandError,
        match=r"You have the Xcode command line tools installed",
    ):
        Xcode.ensure_xcode_is_installed(
            mock_tools,
            xcode_location=default_xcode_install_path,
        )

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )


def test_installed_but_corrupted(xcode, mock_tools):
    """If the Xcode folder exists, but xcodebuild breaks, raise an error."""
    mock_tools.subprocess.check_output.side_effect = [
        xcode + "\n",  # xcode-select -p
        subprocess.CalledProcessError(
            cmd=["xcodebuild", "-version"],
            returncode=1,
            output="Badness occurred",
        ),
    ]

    with pytest.raises(
        BriefcaseCommandError, match=r"should return the current Xcode version"
    ):
        Xcode.ensure_xcode_is_installed(mock_tools, xcode_location=xcode)

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )


def test_installed_no_minimum_version(xcode, mock_tools):
    """If Xcode is installed, but there's no minimum version, check is satisfied."""
    mock_tools.subprocess.check_output.side_effect = [
        xcode + "\n",  # xcode-select -p
        "Xcode 11.2.1\nBuild version 11B500\n",  # xcodebuild -version
    ]

    # Check passes without an error.
    Xcode.ensure_xcode_is_installed(mock_tools, xcode_location=xcode)

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )


def test_installed_extra_output(capsys, xcode, mock_tools):
    """If Xcode but outputs extra content, the check is still satisfied."""
    # This specific output was seen in the wild with Xcode 13.2.1; see #668
    mock_tools.subprocess.check_output.side_effect = [
        xcode + "\n",  # xcode-select -p
        "\n".join(
            [
                "objc[86306]: Class AMSupportURLConnectionDelegate is implemented in both /usr/lib/libauthinstall.dylib (0x20d17ab90) and /Library/Apple/System/Library/PrivateFrameworks/MobileDevice.framework/Versions/A/MobileDevice (0x1084b82c8). One of the two will be used. Which one is undefined."  # noqa: E501
                "objc[86306]: Class AMSupportURLSession is implemented in both /usr/lib/libauthinstall.dylib (0x20d17abe0) and /Library/Apple/System/Library/PrivateFrameworks/MobileDevice.framework/Versions/A/MobileDevice (0x1084b8318). One of the two will be used. Which one is undefined.",  # noqa: E501
                "Xcode 13.2.1",
                "Build version 13C100",
            ]
        ),
    ]

    # Check passes without an error.
    Xcode.ensure_xcode_is_installed(
        mock_tools,
        xcode_location=xcode,
        min_version=(11, 1),
    )

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )
    # No warning generated.
    out = capsys.readouterr().out
    assert "WARNING" not in out


@pytest.mark.parametrize(
    "min_version, version",
    [
        # Exact match
        ((11, 2, 1), "11.2.1"),  # Exact match
        ((11, 2), "11.2.0"),  # Exact match, implied revision.
        ((11,), "11.0.0"),  # Exact match, implied minor version.
        # Rules still work for single digit versions
        ((8, 2, 1), "8.2.1"),  # Exact match
        ((8, 2), "8.2.0"),  # Exact match, implied revision.
        ((8,), "8.0.0"),  # Exact match, implied minor version.
        # Exceeds version
        ((11, 2, 1), "11.2.5"),  # Exceeds revision requirement
        ((11, 2, 1), "11.3.0"),  # Exceeds minor requirement
        ((11, 2, 1), "12.0.0"),  # Exceeds major requirement
        ((11, 2), "11.2.5"),  # Exceeds implied revision requirement
        ((11, 2), "11.3.0"),  # Exceeds minor requirement
        ((11, 2), "12.0.0"),  # Exceeds major requirement
        ((11,), "11.2.5"),  # Exceeds implied revision requirement
        ((11,), "11.3.0"),  # Exceeds implied minor requirement
        ((11,), "12.0.0"),  # Exceeds major requirement
        # 2 digit version number
        # exact match
        ((11, 2, 0), "11.2"),  # Exact match.
        ((11, 2), "11.2"),  # Exact match, implied revision.
        ((11,), "11.2"),  # Exact match, implied minor version.
        # exceeds version
        ((11, 1, 1), "11.2"),  # Exact match.
        ((11, 1), "11.2"),  # Exact match, implied revision.
        ((11,), "11.2"),  # Exact match, implied minor version.
    ],
)
def test_installed_with_minimum_version_success(
    min_version,
    version,
    capsys,
    xcode,
    mock_tools,
):
    """Check Xcode can meet a minimum version requirement."""

    def check_output_mock(cmd_list, *args, **kwargs):
        if cmd_list == ["xcode-select", "-p"]:
            return xcode + "\n"

        if cmd_list == ["xcodebuild", "-version"]:
            return f"Xcode {version}\nBuild version 11B500\n"

        return mock.DEFAULT

    mock_tools.subprocess.check_output.side_effect = check_output_mock

    # Check passes without an error.
    Xcode.ensure_xcode_is_installed(
        mock_tools,
        min_version=min_version,
    )

    # assert xcode-select and xcodebuild were invoked
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )

    # Make sure the warning wasn't displayed.
    out = capsys.readouterr().out
    assert "WARNING" not in out


@pytest.mark.parametrize(
    "min_version, version",
    [
        ((11, 2, 5), "11.2.1"),  # insufficient revision
        ((11, 3), "11.2.1"),  # Insufficient micro version
        ((12,), "11.2.1"),  # Insufficient major version
        ((8, 2, 5), "8.2.1"),  # insufficient revision
        ((8, 3), "8.2.1"),  # Insufficient micro version
        ((9,), "8.2.1"),  # Insufficient major version
    ],
)
def test_installed_with_minimum_version_failure(
    min_version,
    version,
    xcode,
    mock_tools,
):
    """Check Xcode fail to meet a minimum version requirement."""
    mock_tools.subprocess.check_output.side_effect = [
        xcode + "\n",  # xcode-select -p
        f"Xcode {version}\nBuild version 11B500\n",  # xcodebuild -version
    ]

    # Check raises an error.
    with pytest.raises(BriefcaseCommandError):
        Xcode.ensure_xcode_is_installed(
            mock_tools,
            min_version=min_version,
            xcode_location=xcode,
        )

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )


def test_unexpected_version_output(capsys, xcode, mock_tools):
    """If xcodebuild returns unexpected output, assume it's ok..."""
    mock_tools.subprocess.check_output.side_effect = [
        xcode + "\n",  # xcode-select -p
        "Wibble Wibble Wibble\n",  # xcodebuild -version
    ]

    # Check passes without an error...
    Xcode.ensure_xcode_is_installed(
        mock_tools,
        min_version=(11, 2, 1),
        xcode_location=xcode,
    )

    # subprocess was invoked as expected
    mock_tools.subprocess.check_output.assert_has_calls(
        [
            mock.call(["xcode-select", "-p"]),
            mock.call(["xcodebuild", "-version"]),
        ],
        any_order=False,
    )

    # ...but stdout contains a warning
    out = capsys.readouterr().out
    assert "************" in out



import os


def test_dmg_with_installer_icon(package_command, first_app_with_binaries, tmp_path):
    """An installer icon can be specified for a DMG."""
    # Specify an installer icon, and create the matching file.
    first_app_with_binaries.installer_icon = "pretty"
    with open(tmp_path / "base_path/pretty.icns", "wb") as f:
        f.write(b"A pretty installer icon")

    # Package the app without signing or notarization
    package_command.package_app(
        first_app_with_binaries,
        notarize_app=False,
        adhoc_sign=True,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
            "icon": os.fsdecode(tmp_path / "base_path/pretty.icns"),
        },
    )


def test_dmg_with_missing_installer_icon(
    package_command,
    first_app_with_binaries,
    tmp_path,
    capsys,
):
    """If an installer icon is specified, but the specific file is missing, there is a
    warning."""
    # Specify an installer icon, but don't create the matching file.
    first_app_with_binaries.installer_icon = "pretty"
    first_app_with_binaries.packaging_format = "dmg"

    # Package the app without signing or notarization
    package_command.package_app(
        first_app_with_binaries,
        notarize_app=False,
        adhoc_sign=True,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
        },
    )

    # The warning about a missing icon was output
    assert (
        "Can't find pretty.icns to use as DMG installer icon\n"
        in capsys.readouterr().out
    )


def test_dmg_with_app_installer_icon(
    package_command,
    first_app_with_binaries,
    tmp_path,
):
    """An installer will fall back to an app icon for a DMG."""
    # Specify an app icon, and create the matching file.
    first_app_with_binaries.icon = "pretty_app"
    with open(tmp_path / "base_path/pretty_app.icns", "wb") as f:
        f.write(b"A pretty app icon")

    # Package the app without signing or notarization
    package_command.package_app(
        first_app_with_binaries,
        notarize_app=False,
        adhoc_sign=True,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
            "icon": os.fsdecode(tmp_path / "base_path/pretty_app.icns"),
        },
    )


def test_dmg_with_missing_app_installer_icon(
    package_command,
    first_app_with_binaries,
    tmp_path,
    capsys,
):
    """If an app icon is specified, but the specific file is missing, there is a
    warning."""
    # Specify an app icon, but don't create the matching file.
    first_app_with_binaries.icon = "pretty_app"

    # Package the app without signing or notarization
    package_command.package_app(
        first_app_with_binaries,
        notarize_app=False,
        adhoc_sign=True,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
        },
    )

    # The warning about a missing icon was output
    assert (
        "Can't find pretty_app.icns to use as fallback DMG installer icon\n"
        in capsys.readouterr().out
    )


def test_dmg_with_installer_background(
    package_command,
    first_app_with_binaries,
    tmp_path,
):
    """An installer can be built with an installer background."""
    # Specify an installer background, and create the matching file.
    first_app_with_binaries.installer_background = "pretty_background"
    with open(tmp_path / "base_path/pretty_background.png", "wb") as f:
        f.write(b"A pretty background")

    # Package the app without signing or notarization
    package_command.package_app(
        first_app_with_binaries,
        notarize_app=False,
        adhoc_sign=True,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
            "background": os.fsdecode(tmp_path / "base_path/pretty_background.png"),
        },
    )


def test_dmg_with_missing_installer_background(
    package_command,
    first_app_with_binaries,
    tmp_path,
    capsys,
):
    """If an installer image is specified, but the specific file is missing, there is a
    warning."""
    # Specify an installer background, but don't create the matching file.
    first_app_with_binaries.installer_background = "pretty_background"
    first_app_with_binaries.packaging_format = "dmg"

    # Package the app without signing or notarization
    package_command.package_app(
        first_app_with_binaries,
        notarize_app=False,
        adhoc_sign=True,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
        },
    )

    # The warning about a missing background was output
    assert (
        "Can't find pretty_background.png to use as DMG background\n"
        in capsys.readouterr().out
    )



import sys
from unittest.mock import MagicMock, call

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import UnsupportedHostError
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.iOS.xcode import iOSXcodeCreateCommand


@pytest.fixture
def create_command(tmp_path):
    return iOSXcodeCreateCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )


@pytest.mark.parametrize("host_os", ["Linux", "Windows", "WeirdOS"])
def test_unsupported_host_os(create_command, host_os):
    """Error raised for an unsupported OS."""
    create_command.tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match="iOS applications require Xcode, which is only available on macOS.",
    ):
        create_command()


def test_extra_pip_args(create_command, first_app_generated, tmp_path):
    """Extra iOS-specific args are included in calls to pip during update."""
    # Hard code the current architecture for testing. We only install simulator
    # requirements for the current platform.
    create_command.tools.host_arch = "wonky"

    first_app_generated.requires = ["something==1.2.3", "other>=2.3.4"]

    create_command.tools[first_app_generated].app_context = MagicMock(
        spec_set=Subprocess
    )

    create_command.install_app_requirements(first_app_generated, test_mode=False)

    bundle_path = tmp_path / "base_path/build/first-app/ios/xcode"
    assert create_command.tools[first_app_generated].app_context.run.mock_calls == [
        call(
            [
                sys.executable,
                "-u",
                "-X",
                "utf8",
                "-m",
                "pip",
                "install",
                "--disable-pip-version-check",
                "--no-python-version-warning",
                "--upgrade",
                "--no-user",
                f"--target={bundle_path / 'app_packages.iphoneos'}",
                "--prefer-binary",
                "--extra-index-url",
                "https://pypi.anaconda.org/beeware/simple",
                "something==1.2.3",
                "other>=2.3.4",
            ],
            check=True,
            encoding="UTF-8",
            env={
                "PYTHONPATH": str(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "ios"
                    / "xcode"
                    / "support"
                    / "platform-site"
                    / "iphoneos.arm64"
                )
            },
        ),
        call(
            [
                sys.executable,
                "-u",
                "-X",
                "utf8",
                "-m",
                "pip",
                "install",
                "--disable-pip-version-check",
                "--no-python-version-warning",
                "--upgrade",
                "--no-user",
                f"--target={bundle_path / 'app_packages.iphonesimulator'}",
                "--prefer-binary",
                "--extra-index-url",
                "https://pypi.anaconda.org/beeware/simple",
                "something==1.2.3",
                "other>=2.3.4",
            ],
            check=True,
            encoding="UTF-8",
            env={
                "PYTHONPATH": str(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "ios"
                    / "xcode"
                    / "support"
                    / "platform-site"
                    / "iphonesimulator.wonky"
                )
            },
        ),
    ]


@pytest.mark.parametrize(
    "permissions, info, context",
    [
        # No permissions
        (
            {},
            {},
            {"info": {}},
        ),
        # Only custom permissions
        (
            {},
            {
                "NSCameraUsageDescription": "I need to see you",
                "NSMicrophoneUsageDescription": "I need to hear you",
            },
            {
                "info": {
                    "NSCameraUsageDescription": "I need to see you",
                    "NSMicrophoneUsageDescription": "I need to hear you",
                }
            },
        ),
        # Camera permissions
        (
            {
                "camera": "I need to see you",
            },
            {},
            {
                "info": {
                    "NSCameraUsageDescription": "I need to see you",
                },
            },
        ),
        # Microphone permissions
        (
            {
                "microphone": "I need to hear you",
            },
            {},
            {
                "info": {
                    "NSMicrophoneUsageDescription": "I need to hear you",
                },
            },
        ),
        # Coarse location permissions
        (
            {
                "coarse_location": "I need to know roughly where you are",
            },
            {},
            {
                "info": {
                    "NSLocationDefaultAccuracyReduced": True,
                    "NSLocationWhenInUseUsageDescription": "I need to know roughly where you are",
                }
            },
        ),
        # Fine location permissions
        (
            {
                "fine_location": "I need to know exactly where you are",
            },
            {},
            {
                "info": {
                    "NSLocationDefaultAccuracyReduced": False,
                    "NSLocationWhenInUseUsageDescription": "I need to know exactly where you are",
                }
            },
        ),
        # Background location permissions
        (
            {
                "background_location": "I always need to know where you are",
            },
            {},
            {
                "info": {
                    "NSLocationWhenInUseUsageDescription": "I always need to know where you are",
                    "NSLocationAlwaysAndWhenInUseUsageDescription": "I always need to know where you are",
                    "UIBackgroundModes": ["processing", "location"],
                }
            },
        ),
        # Coarse location background permissions
        (
            {
                "coarse_location": "I need to know roughly where you are",
                "background_location": "I always need to know where you are",
            },
            {},
            {
                "info": {
                    "NSLocationDefaultAccuracyReduced": True,
                    "NSLocationWhenInUseUsageDescription": "I need to know roughly where you are",
                    "NSLocationAlwaysAndWhenInUseUsageDescription": "I always need to know where you are",
                    "UIBackgroundModes": ["processing", "location"],
                }
            },
        ),
        # Fine location background permissions
        (
            {
                "fine_location": "I need to know exactly where you are",
                "background_location": "I always need to know where you are",
            },
            {},
            {
                "info": {
                    "NSLocationDefaultAccuracyReduced": False,
                    "NSLocationWhenInUseUsageDescription": "I need to know exactly where you are",
                    "NSLocationAlwaysAndWhenInUseUsageDescription": "I always need to know where you are",
                    "UIBackgroundModes": ["processing", "location"],
                }
            },
        ),
        # Coarse and fine location permissions
        (
            {
                "coarse_location": "I need to know roughly where you are",
                "fine_location": "I need to know exactly where you are",
            },
            {},
            {
                "info": {
                    "NSLocationDefaultAccuracyReduced": False,
                    "NSLocationWhenInUseUsageDescription": "I need to know exactly where you are",
                }
            },
        ),
        # Coarse and fine background location permissions
        (
            {
                "coarse_location": "I need to know roughly where you are",
                "fine_location": "I need to know exactly where you are",
                "background_location": "I always need to know where you are",
            },
            {},
            {
                "info": {
                    "NSLocationDefaultAccuracyReduced": False,
                    "NSLocationWhenInUseUsageDescription": "I need to know exactly where you are",
                    "NSLocationAlwaysAndWhenInUseUsageDescription": "I always need to know where you are",
                    "UIBackgroundModes": ["processing", "location"],
                }
            },
        ),
        # Photo library permissions
        (
            {
                "photo_library": "I need to see your library",
            },
            {},
            {
                "info": {
                    "NSPhotoLibraryAddUsageDescription": "I need to see your library"
                }
            },
        ),
        # Override and augment by cross-platform definitions
        (
            {
                "camera": "I need to see you",
            },
            {
                "NSCameraUsageDescription": "Platform specific",
                "NSCustomPermission": "Custom message",
            },
            {
                "info": {
                    "NSCameraUsageDescription": "Platform specific",
                    "NSCustomPermission": "Custom message",
                }
            },
        ),
    ],
)
def test_permissions_context(create_command, first_app, permissions, info, context):
    """Platform-specific permissions can be added to the context."""
    # Set the permissions value
    first_app.permission = permissions
    first_app.info = info
    # Extract the cross-platform permissions
    x_permissions = create_command._x_permissions(first_app)
    # Check that the final platform permissions are rendered as expected.
    assert context == create_command.permissions_context(first_app, x_permissions)



import pytest


@pytest.mark.parametrize(
    "bundle",
    [
        "com.example",
        "com.example.more",
        "com.example42.more",
        "com.example-42.more",
        "ca.example.issue1212",
        "au.example.issue1212",
        "in.example.issue1212",
        "im.glyph.and.this.is.1212",
    ],
)
def test_valid_bundle(new_command, bundle):
    """Test that valid bundles are accepted."""
    assert new_command.validate_bundle(bundle)


@pytest.mark.parametrize(
    "bundle",
    [
        "not a bundle!",  # Free text.
        "home",  # Only one section.
        "com.hello_world",  # underscore
        "com.hello,world",  # comma
        "com.hello world!",  # exclamation point
    ],
)
def test_invalid_bundle(new_command, bundle):
    """Test that invalid bundles are rejected."""
    with pytest.raises(ValueError):
        new_command.validate_bundle(bundle)



import pytest

from briefcase.console import Console, Log
from briefcase.integrations.base import ToolCache


@pytest.fixture
def simple_tools(tmp_path):
    return ToolCache(logger=Log(), console=Console(), base_path=tmp_path)



import sys
from subprocess import CalledProcessError

import pytest

from briefcase.console import LogLevel
from briefcase.exceptions import RequirementsInstallError


@pytest.mark.parametrize("logging_level", [LogLevel.INFO, LogLevel.DEEP_DEBUG])
def test_install_requirements_no_error(dev_command, first_app, logging_level):
    """Ensure run is executed properly to install requirements."""
    # Configure logging level
    dev_command.logger.verbosity = logging_level

    first_app.requires = ["package-one", "package_two", "packagethree"]

    dev_command.install_dev_requirements(app=first_app)

    dev_command.tools.subprocess.run.assert_called_once_with(
        [
            sys.executable,
            "-u",
            "-X",
            "utf8",
            "-m",
            "pip",
            "install",
            "--upgrade",
        ]
        + (["-vv"] if logging_level == LogLevel.DEEP_DEBUG else [])
        + ["package-one", "package_two", "packagethree"],
        check=True,
        encoding="UTF-8",
    )


def test_install_requirements_error(dev_command, first_app):
    """Ensure RequirementsInstallError exception is raised for install errors."""
    first_app.requires = ["package-one", "package_two", "packagethree"]

    dev_command.tools.subprocess.run.side_effect = CalledProcessError(
        returncode=-1, cmd="pip"
    )

    with pytest.raises(
        RequirementsInstallError,
        match="Unable to install requirements.",
    ):
        dev_command.install_dev_requirements(app=first_app)

    dev_command.tools.subprocess.run.assert_called_once_with(
        [
            sys.executable,
            "-u",
            "-X",
            "utf8",
            "-m",
            "pip",
            "install",
            "--upgrade",
            "package-one",
            "package_two",
            "packagethree",
        ],
        check=True,
        encoding="UTF-8",
    )


def test_no_requirements(dev_command, first_app):
    """Ensure dependency installation is not attempted when nothing to install."""
    first_app.requires = []

    dev_command.install_dev_requirements(app=first_app)

    dev_command.tools.subprocess.run.assert_not_called()


def test_install_requirements_test_mode(dev_command, first_app):
    """If an app has test requirements, they are also installed."""
    first_app.requires = ["package-one", "package_two", "packagethree"]
    first_app.test_requires = ["test-one", "test_two"]

    dev_command.install_dev_requirements(app=first_app)

    dev_command.tools.subprocess.run.assert_called_once_with(
        [
            sys.executable,
            "-u",
            "-X",
            "utf8",
            "-m",
            "pip",
            "install",
            "--upgrade",
            "package-one",
            "package_two",
            "packagethree",
            "test-one",
            "test_two",
        ],
        check=True,
        encoding="UTF-8",
    )


def test_only_test_requirements(dev_command, first_app):
    """If an app only has test requirements, they're installed correctly."""
    first_app.requires = None
    first_app.test_requires = ["test-one", "test_two"]

    dev_command.install_dev_requirements(app=first_app)

    dev_command.tools.subprocess.run.assert_called_once_with(
        [
            sys.executable,
            "-u",
            "-X",
            "utf8",
            "-m",
            "pip",
            "install",
            "--upgrade",
            "test-one",
            "test_two",
        ],
        check=True,
        encoding="UTF-8",
    )



import os
from unittest.mock import MagicMock

import pytest

from briefcase.integrations.base import ToolCache
from briefcase.integrations.subprocess import Subprocess


@pytest.fixture
def mock_tools(tmp_path, mock_tools) -> ToolCache:
    mock_tools.host_os = "Windows"

    mock_tools.os.environ = {
        "ProgramFiles(x86)": os.fsdecode(tmp_path / "Program Files (x86)")
    }

    mock_tools.subprocess = Subprocess(mock_tools)
    mock_tools.subprocess.check_output = MagicMock()

    return mock_tools



import os
from unittest.mock import MagicMock

import pytest

from briefcase.commands import OpenCommand
from briefcase.commands.base import full_options
from briefcase.config import AppConfig
from briefcase.console import Console, Log
from briefcase.integrations.subprocess import Subprocess

from ...utils import create_file


class DummyOpenCommand(OpenCommand):
    """A dummy open command that doesn't actually do anything.

    It only serves to track which actions would be performed.
    """

    # Platform and format contain upper case to test case normalization
    platform = "Tester"
    output_format = "Dummy"
    description = "Dummy Open command"

    def __init__(self, *args, apps, **kwargs):
        kwargs.setdefault("logger", Log())
        kwargs.setdefault("console", Console())
        super().__init__(*args, apps=apps, **kwargs)

        # Override the OS services that are used when opening
        self.tools.os = MagicMock(spec_set=os)
        self.tools.subprocess = MagicMock(spec_set=Subprocess)

        self.actions = []

    def briefcase_toml(self, app):
        # default any app to an empty `briefcase.toml`
        return self._briefcase_toml.get(app, {})

    def binary_path(self, app):
        raise NotImplementedError(
            "Required by interface contract, but should not be used"
        )

    def project_path(self, app):
        return self.bundle_path(app) / f"{app.formal_name}.project"

    def verify_host(self):
        super().verify_host()
        self.actions.append(("verify-host",))

    def verify_tools(self):
        super().verify_tools()
        self.actions.append(("verify-tools",))

    def finalize_app_config(self, app):
        super().finalize_app_config(app=app)
        self.actions.append(("finalize-app-config", app.app_name))

    def verify_app_template(self, app):
        super().verify_app_template(app=app)
        self.actions.append(("verify-app-template", app.app_name))

    def verify_app_tools(self, app):
        super().verify_app_tools(app=app)
        self.actions.append(("verify-app-tools", app.app_name))

    def _open_app(self, app):
        super()._open_app(app)
        self.actions.append(("open", app.app_name))

    # These commands override the default behavior, simply tracking that
    # they were invoked, rather than instantiating a Create command.
    # This is for testing purposes.
    def create_command(self, app, **kwargs):
        self.actions.append(("create", app.app_name, kwargs))
        return full_options({"create_state": app.app_name}, kwargs)


@pytest.fixture
def open_command(tmp_path):
    return DummyOpenCommand(
        base_path=tmp_path / "base_path",
        apps={
            "first": AppConfig(
                app_name="first",
                bundle="com.example",
                version="0.0.1",
                description="The first simple app",
                sources=["src/first"],
                license={"file": "LICENSE"},
            ),
            "second": AppConfig(
                app_name="second",
                bundle="com.example",
                version="0.0.2",
                description="The second simple app",
                sources=["src/second"],
                license={"file": "LICENSE"},
            ),
        },
    )


@pytest.fixture
def first_app(tmp_path):
    """Populate skeleton app content for the first app."""
    project_file = (
        tmp_path
        / "base_path"
        / "build"
        / "first"
        / "tester"
        / "dummy"
        / "first.project"
    )
    create_file(project_file, "first project")
    return project_file


@pytest.fixture
def second_app(tmp_path):
    """Populate skeleton app content for the second app."""
    project_file = (
        tmp_path
        / "base_path"
        / "build"
        / "second"
        / "tester"
        / "dummy"
        / "second.project"
    )
    create_file(project_file, "second project")
    return project_file



import os
import subprocess
from pathlib import Path
from subprocess import CalledProcessError
from unittest.mock import ANY

import pytest

from briefcase.console import LogLevel

from .conftest import CREATE_NEW_PROCESS_GROUP, CREATE_NO_WINDOW


@pytest.mark.parametrize("platform", ["Linux", "Darwin", "Windows"])
def test_call(mock_sub, capsys, platform, sub_check_output_kw):
    """A simple call will be invoked."""

    mock_sub.tools.host_os = platform
    mock_sub.check_output(["hello", "world"])

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        **sub_check_output_kw,
    )
    assert capsys.readouterr().out == ""


def test_call_with_arg(mock_sub, capsys, sub_check_output_kw):
    """Any extra keyword arguments are passed through as-is."""

    mock_sub.check_output(["hello", "world"], extra_kw="extra")

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        extra_kw="extra",
        **sub_check_output_kw,
    )
    assert capsys.readouterr().out == ""


def test_call_with_path_arg(mock_sub, capsys, tmp_path, sub_check_output_kw):
    """Path-based arguments are converted to strings and passed in as-is."""

    mock_sub.check_output(["hello", tmp_path / "location"], cwd=tmp_path / "cwd")

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", os.fsdecode(tmp_path / "location")],
        cwd=os.fsdecode(tmp_path / "cwd"),
        **sub_check_output_kw,
    )
    assert capsys.readouterr().out == ""


@pytest.mark.parametrize(
    ("platform", "start_new_session", "check_output_kwargs"),
    [
        ("Linux", None, {}),
        ("Linux", True, {}),
        ("Linux", False, {}),
        ("Darwin", None, {}),
        ("Darwin", True, {}),
        ("Darwin", False, {}),
        ("Windows", None, {}),
        (
            "Windows",
            True,
            {"creationflags": CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW},
        ),
        ("Windows", False, {}),
    ],
)
def test_call_with_start_new_session(
    mock_sub,
    capsys,
    platform,
    start_new_session,
    check_output_kwargs,
    sub_check_output_kw,
):
    """start_new_session is passed thru on Linux and macOS but converted for Windows."""

    mock_sub.tools.host_os = platform
    mock_sub.check_output(["hello", "world"], start_new_session=start_new_session)

    final_kwargs = {**check_output_kwargs, **sub_check_output_kw}

    if platform == "Windows":
        mock_sub._subprocess.check_output.assert_called_with(
            ["hello", "world"],
            **final_kwargs,
        )
        assert capsys.readouterr().out == ""
    else:
        mock_sub._subprocess.check_output.assert_called_with(
            ["hello", "world"],
            start_new_session=start_new_session,
            **final_kwargs,
        )
        assert capsys.readouterr().out == ""


@pytest.mark.parametrize(
    ("creationflags", "final_creationflags"),
    [
        (0x1, CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW | 1),
        (CREATE_NEW_PROCESS_GROUP, CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW),
        (0, CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW),
    ],
)
def test_call_windows_with_start_new_session_and_creationflags(
    mock_sub,
    capsys,
    creationflags,
    final_creationflags,
):
    """Creationflags used to simulate start_new_session=True should be merged with any
    existing flags."""

    mock_sub.tools.host_os = "Windows"

    # use commented test below when merging creationflags is allowed
    with pytest.raises(
        AssertionError, match="Subprocess called with creationflags set"
    ):
        mock_sub.check_output(
            ["hello", "world"],
            start_new_session=True,
            creationflags=creationflags,
        )


def test_debug_call(mock_sub, capsys, sub_check_output_kw):
    """If verbosity is turned up, there is output."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    mock_sub.check_output(["hello", "world"])

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        **sub_check_output_kw,
    )

    expected_output = (
        "\n"
        ">>> Running Command:\n"
        ">>>     hello world\n"
        ">>> Working Directory:\n"
        f">>>     {Path.cwd()}\n"
        ">>> Command Output:\n"
        ">>>     some output line 1\n"
        ">>>     more output line 2\n"
        ">>> Return code: 0\n"
    )

    assert capsys.readouterr().out == expected_output


def test_debug_call_with_env(mock_sub, capsys, tmp_path, sub_check_output_kw):
    """If verbosity is turned up, injected env vars are included in output."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    env = {"NewVar": "NewVarValue"}
    mock_sub.check_output(["hello", "world"], env=env, cwd=tmp_path / "cwd")

    merged_env = mock_sub.tools.os.environ.copy()
    merged_env.update(env)

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        env=merged_env,
        cwd=os.fsdecode(tmp_path / "cwd"),
        **sub_check_output_kw,
    )

    expected_output = (
        "\n"
        ">>> Running Command:\n"
        ">>>     hello world\n"
        ">>> Working Directory:\n"
        f">>>     {tmp_path / 'cwd'}\n"
        ">>> Environment Overrides:\n"
        ">>>     NewVar=NewVarValue\n"
        ">>> Command Output:\n"
        ">>>     some output line 1\n"
        ">>>     more output line 2\n"
        ">>> Return code: 0\n"
    )

    assert capsys.readouterr().out == expected_output


def test_debug_call_with_quiet(mock_sub, capsys, tmp_path, sub_check_output_kw):
    """If quiet mode is on, calls aren't logged, even if verbosity is turned up."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    env = {"NewVar": "NewVarValue"}
    mock_sub.check_output(
        ["hello", "world"],
        env=env,
        cwd=tmp_path / "cwd",
        quiet=True,
    )

    merged_env = mock_sub.tools.os.environ.copy()
    merged_env.update(env)

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        env=merged_env,
        cwd=os.fsdecode(tmp_path / "cwd"),
        **sub_check_output_kw,
    )

    # No output
    assert capsys.readouterr().out == ""


def test_debug_call_with_stderr(mock_sub, capsys, tmp_path, sub_check_output_kw):
    """If stderr is specified, it is not defaulted to stdout."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    mock_sub.check_output(
        ["hello", "world"],
        cwd=tmp_path / "cwd",
        stderr=subprocess.DEVNULL,
    )

    sub_check_output_kw.pop("stderr")
    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        cwd=os.fsdecode(tmp_path / "cwd"),
        stderr=subprocess.DEVNULL,
        **sub_check_output_kw,
    )

    expected_output = (
        "\n"
        ">>> Running Command:\n"
        ">>>     hello world\n"
        ">>> Working Directory:\n"
        f">>>     {tmp_path / 'cwd'}\n"
        ">>> Command Output:\n"
        ">>>     some output line 1\n"
        ">>>     more output line 2\n"
        ">>> Return code: 0\n"
    )

    assert capsys.readouterr().out == expected_output


def test_calledprocesserror_exception_logging(mock_sub, capsys):
    """If command errors, ensure command output is printed."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    called_process_error = CalledProcessError(
        returncode=-1,
        cmd="hello world",
        output="output line 1\noutput line 2",
        stderr="error line 1\nerror line 2",
    )
    mock_sub._subprocess.check_output.side_effect = called_process_error

    with pytest.raises(CalledProcessError):
        mock_sub.check_output(["hello", "world"])

    expected_output = (
        "\n"
        ">>> Running Command:\n"
        ">>>     hello world\n"
        ">>> Working Directory:\n"
        f">>>     {Path.cwd()}\n"
        ">>> Command Output:\n"
        ">>>     output line 1\n"
        ">>>     output line 2\n"
        ">>> Command Error Output (stderr):\n"
        ">>>     error line 1\n"
        ">>>     error line 2\n"
        ">>> Return code: -1\n"
    )

    assert capsys.readouterr().out == expected_output


def test_calledprocesserror_exception_quiet(mock_sub, capsys):
    """If command errors in quiet mode, no command output is printed."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    called_process_error = CalledProcessError(
        returncode=-1,
        cmd="hello world",
        output="output line 1\noutput line 2",
        stderr="error line 1\nerror line 2",
    )
    mock_sub._subprocess.check_output.side_effect = called_process_error

    with pytest.raises(CalledProcessError):
        mock_sub.check_output(["hello", "world"], quiet=True)

    # No output in quiet mode
    assert capsys.readouterr().out == ""


def test_calledprocesserror_exception_logging_no_cmd_output(mock_sub, capsys):
    """If command errors, and there is no command output, errors are still printed."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    called_process_error = CalledProcessError(
        returncode=-1,
        cmd="hello world",
        output=None,
        stderr="error line 1\nerror line 2",
    )
    mock_sub._subprocess.check_output.side_effect = called_process_error

    with pytest.raises(CalledProcessError):
        mock_sub.check_output(["hello", "world"])

    expected_output = (
        "\n"
        ">>> Running Command:\n"
        ">>>     hello world\n"
        ">>> Working Directory:\n"
        f">>>     {Path.cwd()}\n"
        ">>> Command Error Output (stderr):\n"
        ">>>     error line 1\n"
        ">>>     error line 2\n"
        ">>> Return code: -1\n"
    )

    assert capsys.readouterr().out == expected_output


@pytest.mark.parametrize(
    "in_kwargs, kwargs",
    [
        (
            {},
            {
                "text": True,
                "encoding": ANY,
                "stderr": subprocess.STDOUT,
                "errors": "backslashreplace",
            },
        ),
        (
            {"text": True},
            {
                "text": True,
                "encoding": ANY,
                "stderr": subprocess.STDOUT,
                "errors": "backslashreplace",
            },
        ),
        ({"text": False}, {"text": False, "stderr": subprocess.STDOUT}),
        (
            {"universal_newlines": False},
            {"text": False, "stderr": subprocess.STDOUT},
        ),
        (
            {"universal_newlines": True},
            {
                "text": True,
                "encoding": ANY,
                "stderr": subprocess.STDOUT,
                "errors": "backslashreplace",
            },
        ),
    ],
)
def test_text_eq_true_default_overriding(mock_sub, in_kwargs, kwargs):
    """If text or universal_newlines is explicitly provided, those should override
    text=true default and universal_newlines should be converted to text."""
    mock_sub.check_output(["hello", "world"], stderr=subprocess.STDOUT, **in_kwargs)

    mock_sub._subprocess.check_output.assert_called_with(["hello", "world"], **kwargs)



import pytest

from briefcase.commands import UpdateCommand
from briefcase.config import AppConfig
from briefcase.console import Console, Log

from ...utils import create_file


class DummyUpdateCommand(UpdateCommand):
    """A dummy update command that doesn't actually do anything.

    It only serves to track which actions would be performed.
    """

    # Platform and format contain upper case to test case normalization
    platform = "Tester"
    output_format = "Dummy"
    description = "Dummy update command"

    def __init__(self, *args, apps, **kwargs):
        kwargs.setdefault("logger", Log())
        kwargs.setdefault("console", Console())
        super().__init__(*args, apps=apps, **kwargs)

        self.actions = []

    def briefcase_toml(self, app):
        # default any app to an empty `briefcase.toml`
        return self._briefcase_toml.get(app, {})

    def binary_path(self, app):
        return self.bundle_path(app) / f"{app.app_name}.bin"

    def verify_host(self):
        super().verify_host()
        self.actions.append(("verify-host",))

    def verify_tools(self):
        super().verify_tools()
        self.actions.append(("verify-tools",))

    def finalize_app_config(self, app):
        super().finalize_app_config(app=app)
        self.actions.append(("finalize-app-config", app.app_name))

    def verify_app_template(self, app):
        super().verify_app_template(app=app)
        self.actions.append(("verify-app-template", app.app_name))

    def verify_app_tools(self, app):
        super().verify_app_tools(app=app)
        self.actions.append(("verify-app-tools", app.app_name))

    # Override all the body methods of a UpdateCommand
    # with versions that we can use to track actions performed.
    def install_app_requirements(self, app, test_mode):
        self.actions.append(("requirements", app.app_name, test_mode))
        create_file(self.bundle_path(app) / "requirements", "app requirements")

    def install_app_code(self, app, test_mode):
        self.actions.append(("code", app.app_name, test_mode))
        create_file(self.bundle_path(app) / "code.py", "print('app')")

    def install_app_resources(self, app):
        self.actions.append(("resources", app.app_name))
        create_file(self.bundle_path(app) / "resources", "app resources")

    def cleanup_app_support_package(self, app):
        self.actions.append(("cleanup-support", app.app_name))

    def install_app_support_package(self, app):
        self.actions.append(("support", app.app_name))
        create_file(self.bundle_path(app) / "support/content.txt", "app support")

    def cleanup_stub_binary(self, app):
        self.actions.append(("cleanup-stub", app.app_name))

    def install_stub_binary(self, app):
        self.actions.append(("stub", app.app_name))
        create_file(self.bundle_path(app) / "stub.exe", "app stub")

    def cleanup_app_content(self, app):
        self.actions.append(("cleanup", app.app_name))


@pytest.fixture
def update_command(tmp_path):
    return DummyUpdateCommand(
        base_path=tmp_path / "base_path",
        apps={
            "first": AppConfig(
                app_name="first",
                bundle="com.example",
                version="0.0.1",
                description="The first simple app",
                sources=["src/first"],
                license={"file": "LICENSE"},
            ),
            "second": AppConfig(
                app_name="second",
                bundle="com.example",
                version="0.0.2",
                description="The second simple app",
                sources=["src/second"],
                license={"file": "LICENSE"},
            ),
        },
    )


@pytest.fixture
def first_app(tmp_path):
    """Populate skeleton app content for the first app."""
    create_file(
        tmp_path
        / "base_path"
        / "build"
        / "first"
        / "tester"
        / "dummy"
        / "first.bundle",
        "first.bundle",
    )


@pytest.fixture
def second_app(tmp_path):
    """Populate skeleton app content for the second app."""
    create_file(
        tmp_path
        / "base_path"
        / "build"
        / "second"
        / "tester"
        / "dummy"
        / "second.bundle",
        "second.bundle",
    )



from ...utils import create_installed_package


def test_find_binary_packages(dummy_command, tmp_path):
    """Binary packages can be identified in a app-packages folder."""

    create_installed_package(
        tmp_path / "app-packages",
        "pure-package1",
        version="1.2.3",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "pure-package2",
        version="1.2.4",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "universal-package-1",
        version="2.3.4",
        tag="macOS_11_universal2",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "universal-package-2",
        version="2.3.5",
        tag="macOS_13_universal2",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "binary-package-1",
        version="3.4.5",
        tag="macOS_11_arm64",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "binary-package-2",
        version="3.4.6",
        tag="macOS_13_arm64",
    )

    binary_packages = dummy_command.find_binary_packages(
        tmp_path / "app-packages",
        universal_suffix="_universal2",
    )

    # Binary wheels are discovered. We don't care about the order they are returned in,
    # just that they're all found.
    assert len(binary_packages) == 2
    assert set(binary_packages) == {
        ("binary-package-1", "3.4.5"),
        ("binary-package-2", "3.4.6"),
    }


def test_find_binary_packages_non_universal(dummy_command, tmp_path):
    """If no universal wheel format is specified, universal wheels are identified as
    binary."""

    create_installed_package(
        tmp_path / "app-packages",
        "pure-package1",
        version="1.2.3",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "pure-package2",
        version="1.2.4",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "universal-package-1",
        version="2.3.4",
        tag="macOS_11_universal2",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "universal-package-2",
        version="2.3.5",
        tag="macOS_13_universal2",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "binary-package-1",
        version="3.4.5",
        tag="macOS_11_arm64",
    )
    create_installed_package(
        tmp_path / "app-packages",
        "binary-package-2",
        version="3.4.6",
        tag="macOS_13_arm64",
    )

    binary_packages = dummy_command.find_binary_packages(tmp_path / "app-packages")

    # Binary wheels are discovered. We don't care about the order they are returned in,
    # just that they're all found.
    assert len(binary_packages) == 4
    assert set(binary_packages) == {
        ("universal-package-1", "2.3.4"),
        ("universal-package-2", "2.3.5"),
        ("binary-package-1", "3.4.5"),
        ("binary-package-2", "3.4.6"),
    }



import pytest
import tomli_w

from briefcase.exceptions import BriefcaseCommandError


def test_template_url(create_command):
    """The template URL is a simple construction of the platform and format."""
    assert (
        create_command.app_template_url
        == "https://github.com/beeware/briefcase-Tester-Dummy-template.git"
    )


def test_missing_briefcase_toml(create_command, myapp):
    with pytest.raises(BriefcaseCommandError, match=r"Unable to find.*briefcase.toml'"):
        _ = create_command.app_path(myapp)


def test_app_path(create_command, myapp):
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "app_packages_path": "path/to/app_packages",
                "support_path": "path/to/support",
            }
        }
        tomli_w.dump(index, f)

    assert create_command.app_path(myapp) == bundle_path / "path/to/app"

    # Requesting a second time should hit the cache,
    # so the briefcase file won't be needed.
    # Delete it to make sure the cache is used.
    (bundle_path / "briefcase.toml").unlink()
    assert create_command.app_path(myapp) == bundle_path / "path/to/app"


def test_app_packages_path(create_command, myapp):
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "app_packages_path": "path/to/app_packages",
                "support_path": "path/to/support",
            }
        }
        tomli_w.dump(index, f)

    assert (
        create_command.app_packages_path(myapp) == bundle_path / "path/to/app_packages"
    )

    # Requesting a second time should hit the cache,
    # so the briefcase file won't be needed.
    # Delete it to make sure the cache is used.
    (bundle_path / "briefcase.toml").unlink()
    assert (
        create_command.app_packages_path(myapp) == bundle_path / "path/to/app_packages"
    )


def test_support_path(create_command, myapp):
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "app_packages_path": "path/to/app_packages",
                "support_path": "path/to/support",
            }
        }
        tomli_w.dump(index, f)

    assert create_command.support_path(myapp) == bundle_path / "path/to/support"

    # Requesting a second time should hit the cache,
    # so the briefcase file won't be needed.
    # Delete it to make sure the cache is used.
    (bundle_path / "briefcase.toml").unlink()
    assert create_command.support_path(myapp) == bundle_path / "path/to/support"


def test_support_revision(create_command, myapp):
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "app_packages_path": "path/to/app_packages",
                "support_path": "path/to/support",
                "support_revision": 42,
            }
        }
        tomli_w.dump(index, f)

    assert create_command.support_revision(myapp) == 42

    # Requesting a second time should hit the cache,
    # so the briefcase file won't be needed.
    # Delete it to make sure the cache is used.
    (bundle_path / "briefcase.toml").unlink()
    assert create_command.support_revision(myapp) == 42


def test_cleanup_paths(create_command, myapp):
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "cleanup_paths": ["path/to/glob*", "other/path"],
            }
        }
        tomli_w.dump(index, f)

    assert create_command.cleanup_paths(myapp) == ["path/to/glob*", "other/path"]

    # Requesting a second time should hit the cache,
    # so the briefcase file won't be needed.
    # Delete it to make sure the cache is used.
    (bundle_path / "briefcase.toml").unlink()
    assert create_command.cleanup_paths(myapp) == ["path/to/glob*", "other/path"]


def test_support_package_url(create_command):
    # Retrieve the property, retrieving the support package URL.
    url = "https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/Python-3.X-Tester-support.b52.tar.gz"
    assert create_command.support_package_url(52) == url


def test_no_icon(create_command, myapp):
    """If no icon target is specified, the icon list is empty."""
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
            }
        }
        tomli_w.dump(index, f)

    assert create_command.icon_targets(myapp) == {}


def test_single_icon(create_command, myapp):
    """If the icon target is specified as a single string, the icon list has one unsized
    entry."""
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "icon": "path/to/icon.png",
            }
        }
        tomli_w.dump(index, f)

    assert create_command.icon_targets(myapp) == {None: "path/to/icon.png"}


def test_multiple_icon(create_command, myapp):
    """If there are multiple icon targets, they're all in the target list."""
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "icon": {
                    "10": "path/to/icon-10.png",
                    "20": "path/to/icon-20.png",
                },
            }
        }
        tomli_w.dump(index, f)

    assert create_command.icon_targets(myapp) == {
        "10": "path/to/icon-10.png",
        "20": "path/to/icon-20.png",
    }


def test_icon_variants(create_command, myapp):
    """If there are icon variants, they are returned."""
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "icon": {
                    "single": "path/to/icon.png",
                    "multi": {
                        "10": "path/to/icon-10.png",
                        "20": "path/to/icon-20.png",
                    },
                },
            }
        }
        tomli_w.dump(index, f)

    assert create_command.icon_targets(myapp) == {
        "single": "path/to/icon.png",
        "multi": {
            "10": "path/to/icon-10.png",
            "20": "path/to/icon-20.png",
        },
    }


def test_no_document_types(create_command, myapp):
    """If no document type targets are specified, the document_type_icons list is
    empty."""
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
            }
        }
        tomli_w.dump(index, f)

    assert create_command.document_type_icon_targets(myapp) == {}


def test_document_type_single_icon(create_command, myapp):
    """If a doctype icon target is specified as a single string, the document_type_icons
    list has one unsized entry."""
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "document_type_icon": {
                    "mydoc": "path/to/mydoc-icon.png",
                    "other": "path/to/otherdoc-icon.png",
                },
            }
        }
        tomli_w.dump(index, f)

    assert create_command.document_type_icon_targets(myapp) == {
        "mydoc": {
            None: "path/to/mydoc-icon.png",
        },
        "other": {
            None: "path/to/otherdoc-icon.png",
        },
    }


def test_document_type_multiple_icons(create_command, myapp):
    """If there are multiple document_type_icons targets, they're all in the target
    list."""
    bundle_path = create_command.bundle_path(myapp)
    bundle_path.mkdir(parents=True)
    with (bundle_path / "briefcase.toml").open("wb") as f:
        index = {
            "paths": {
                "app_path": "path/to/app",
                "document_type_icon": {
                    "mydoc": "path/to/mydoc-icon.png",
                    "other": {
                        "10": "path/to/otherdoc-icon-10.png",
                        "20": "path/to/otherdoc-icon-20.png",
                    },
                },
            }
        }
        tomli_w.dump(index, f)

    assert create_command.document_type_icon_targets(myapp) == {
        "mydoc": {
            None: "path/to/mydoc-icon.png",
        },
        "other": {
            "10": "path/to/otherdoc-icon-10.png",
            "20": "path/to/otherdoc-icon-20.png",
        },
    }


def test_default_output_format_template_context(default_create_command, myapp):
    """The default output format template context is empty."""
    assert default_create_command.output_format_template_context(myapp) == {}



from briefcase.platforms.macOS.utils import sha256_file_digest

from ...utils import create_file


def test_sha256_file_digest(tmp_path):
    """A sha256 digest can be computed for a file."""
    # Create a file
    create_file(tmp_path / "content.txt", b"Hello world\nGoodbye world.\n", mode="wb")

    # A sha256 digest is returned.
    assert sha256_file_digest(tmp_path / "content.txt") == (
        "8c9b99fbb457c5dfc4553ef9ba9e7e2c55baaaf2fec35897c4bfb4fd2895a5f7"
    )



import pytest

from briefcase.integrations.linuxdeploy import LinuxDeployURLPlugin


@pytest.fixture
def linuxdeploy_url_plugin(mock_tools):
    return LinuxDeployURLPlugin(
        mock_tools,
        url="https://example.com/path/to/linuxdeploy-plugin-foobar.sh",
    )


def test_file_path(mock_tools, linuxdeploy_url_plugin):
    """Custom URL plugins are in the linuxdeploy plugins folder, behind a hash."""
    assert (
        linuxdeploy_url_plugin.file_path
        == mock_tools.base_path
        / "linuxdeploy_plugins"
        / "foobar"
        / "dc66c26aaeb8083777d1975e55dfb5c197b5b54e7b46481793eab4b3f2ace1b3"
    )


def test_file_name(linuxdeploy_url_plugin):
    """Custom URL plugin filenames come from the URL."""
    assert linuxdeploy_url_plugin.file_name == "linuxdeploy-plugin-foobar.sh"


def test_plugin_id(linuxdeploy_url_plugin):
    """The Custom URL plugin ID can be determined from the filename."""
    assert linuxdeploy_url_plugin.plugin_id == "foobar"


def test_download_url(linuxdeploy_url_plugin):
    """The download URL for the plugin is as-provided."""
    assert linuxdeploy_url_plugin.download_url == (
        "https://example.com/path/to/linuxdeploy-plugin-foobar.sh"
    )



import subprocess
from unittest.mock import MagicMock

import pytest

from briefcase.exceptions import BriefcaseCommandError, InvalidDeviceError


def test_emulator(adb, capsys):
    """Invoking `avd_name()` on an emulator returns the AVD."""
    # Mock out the adb response for an emulator
    adb.run = MagicMock(return_value="exampledevice\nOK\n")

    # Invoke avd_name
    assert adb.avd_name() == "exampledevice"

    # Validate call parameters.
    adb.run.assert_called_once_with("emu", "avd", "name")


def test_device(adb, capsys):
    """Invoking `avd_name()` on a device returns None."""
    # Mock out the adb response for a physical device
    adb.run = MagicMock(
        side_effect=subprocess.CalledProcessError(returncode=1, cmd="emu avd name")
    )

    # Invoke avd_name
    assert adb.avd_name() is None

    # Validate call parameters.
    adb.run.assert_called_once_with("emu", "avd", "name")


def test_adb_failure(adb, capsys):
    """If `adb()` fails for a miscellaneous reason, an error is raised."""
    # Mock out the run command on an adb instance
    adb.run = MagicMock(
        side_effect=subprocess.CalledProcessError(returncode=69, cmd="emu avd name")
    )

    # Invoke install
    with pytest.raises(BriefcaseCommandError):
        adb.avd_name()

    # Validate call parameters.
    adb.run.assert_called_once_with("emu", "avd", "name")


def test_invalid_device(adb, capsys):
    """Invoking `avd_name()` on an invalid device raises an error."""
    # Mock out the run command on an adb instance
    adb.run = MagicMock(side_effect=InvalidDeviceError("device", "exampleDevice"))

    # Invoke install
    with pytest.raises(InvalidDeviceError):
        adb.avd_name()

    # Validate call parameters.
    adb.run.assert_called_once_with("emu", "avd", "name")



import pytest

from briefcase.exceptions import BriefcaseCommandError


def test_no_args_one_app(run_command, first_app):
    """If there is one app, run starts that app by default."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure no command line options
    options, _ = run_command.parse_options([])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Run the first app
        ("run", "first", {"test_mode": False, "passthrough": []}),
    ]


def test_no_args_one_app_with_passthrough(run_command, first_app):
    """If there is one app, run starts that app by default, and can accept
    passthrough."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure no command line options
    options, _ = run_command.parse_options(["--", "foo", "--bar"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools have been verified
        ("verify-app-tools", "first"),
        # Run the first app
        ("run", "first", {"test_mode": False, "passthrough": ["foo", "--bar"]}),
    ]


def test_no_args_two_apps(run_command, first_app, second_app):
    """If there are two apps and no explicit app is started, an error is raised."""
    # Add two apps
    run_command.apps = {
        "first": first_app,
        "second": second_app,
    }

    # Configure no command line options
    options, _ = run_command.parse_options([])

    # Invoking the run command raises an error
    with pytest.raises(BriefcaseCommandError):
        run_command(**options)

    # No verification actions will be performed
    assert run_command.actions == []


def test_with_arg_one_app(run_command, first_app):
    """If there is one app, and a -a argument, run starts that app."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure a -a command line option
    options, _ = run_command.parse_options(["-a", "first"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Run the first app
        ("run", "first", {"test_mode": False, "passthrough": []}),
    ]


def test_with_arg_two_apps(run_command, first_app, second_app):
    """If there are multiple apps, the --app argument starts app nominated."""
    # Add two apps
    run_command.apps = {
        "first": first_app,
        "second": second_app,
    }

    # Configure a --app command line option
    options, _ = run_command.parse_options(["--app", "second"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "second"),
        # App template is verified
        ("verify-app-template", "second"),
        # App tools have been verified
        ("verify-app-tools", "second"),
        # Run the second app
        ("run", "second", {"test_mode": False, "passthrough": []}),
    ]


def test_bad_app_reference(run_command, first_app, second_app):
    """If the command line argument refers to an app that doesn't exist, raise an
    error."""
    # Add two apps
    run_command.apps = {
        "first": first_app,
        "second": second_app,
    }

    # Configure a --app command line option
    options, _ = run_command.parse_options(["--app", "does-not-exist"])

    # Invoking the run command raises an error
    with pytest.raises(BriefcaseCommandError):
        run_command(**options)

    # No verification actions will be performed
    assert run_command.actions == []


def test_create_app_before_start(run_command, first_app_config):
    """If the app to be started doesn't exist, create it first."""
    # Add a single app, using the 'config only' fixture
    run_command.apps = {
        "first": first_app_config,
    }

    # Configure no command line options
    options, _ = run_command.parse_options([])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App doesn't exist, so it will be built
        # (which will transitively create)
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": False,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_build_app_before_start(run_command, first_app_unbuilt):
    """The run command can request that an unbuilt app is compiled first."""
    # Add a single app
    run_command.apps = {
        "first": first_app_unbuilt,
    }

    # Configure no command line options
    options, _ = run_command.parse_options([])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # A build was requested, with no update
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": False,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_update_app(run_command, first_app):
    """The run command can request that the app is updated first."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure an update option
    options, _ = run_command.parse_options(["-u"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # A build with an explicit update was requested
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": True,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_update_app_requirements(run_command, first_app):
    """The run command can request that the app requirements are updated first."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure an update option
    options, _ = run_command.parse_options(["-r"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # A build with an explicit update was requested
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": False,
                "update_requirements": True,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_update_app_resources(run_command, first_app):
    """The run command can request that the app resources are updated first."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure an update option
    options, _ = run_command.parse_options(["--update-resources"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # A build with an explicit update was requested
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": False,
                "update_requirements": False,
                "update_resources": True,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_update_app_support(run_command, first_app):
    """The run command can request that the app support is updated first."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure an update option
    options, _ = run_command.parse_options(["--update-support"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # A build with an explicit update was requested
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": False,
                "update_requirements": False,
                "update_resources": False,
                "update_support": True,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_update_app_stub(run_command, first_app):
    """The run command can request that the app stub is updated first."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure an update option
    options, _ = run_command.parse_options(["--update-stub"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # A build with an explicit update was requested
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": False,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": True,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_update_unbuilt_app(run_command, first_app_unbuilt):
    """The run command can request that an unbuilt app is updated first."""
    # Add a single app
    run_command.apps = {
        "first": first_app_unbuilt,
    }

    # Configure an update option
    options, _ = run_command.parse_options(["-u"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # An update was requested, so a build with an explicit update
        # will be performed
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": True,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_update_non_existent(run_command, first_app_config):
    """Requesting an update of a non-existent app causes a create."""
    # Add a single app, using the 'config only' fixture
    run_command.apps = {
        "first": first_app_config,
    }

    # Configure an update option
    options, _ = run_command.parse_options(["-u"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App doesn't exist, so it will be built, with an
        # update requested
        (
            "build",
            "first",
            {
                "test_mode": False,
                "update": True,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": False, "passthrough": []},
        ),
    ]


def test_test_mode_existing_app(run_command, first_app):
    """An existing app can be started in test mode."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure the test option
    options, _ = run_command.parse_options(["--test"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App is built in test mode
        (
            "build",
            "first",
            {
                "test_mode": True,
                "update": False,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Run the first app
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": True, "passthrough": []},
        ),
    ]


def test_test_mode_existing_app_with_passthrough(run_command, first_app):
    """An existing app can be started in test mode with passthrough args."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure the test option
    options, _ = run_command.parse_options(["--test", "--", "foo", "--bar"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App is built in test mode
        (
            "build",
            "first",
            {
                "test_mode": True,
                "update": False,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools have been verified
        ("verify-app-tools", "first"),
        # Run the first app
        (
            "run",
            "first",
            {
                "build_state": "first",
                "test_mode": True,
                "passthrough": ["foo", "--bar"],
            },
        ),
    ]


def test_test_mode_existing_app_no_update(run_command, first_app):
    """The auto app update implied by --test can be overridden."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure the test option
    options, _ = run_command.parse_options(["--test", "--no-update"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App will not be built; update is disabled
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Run the first app
        (
            "run",
            "first",
            {"test_mode": True, "passthrough": []},
        ),
    ]


def test_test_mode_existing_app_update_requirements(run_command, first_app):
    """Requirements can be updated for a test run."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure the test option
    options, _ = run_command.parse_options(["--test", "--update-requirements"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App will be built with a requirements update
        (
            "build",
            "first",
            {
                "test_mode": True,
                "update": False,
                "update_requirements": True,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Run the first app
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": True, "passthrough": []},
        ),
    ]


def test_test_mode_existing_app_update_resources(run_command, first_app):
    """Resources can be updated in test mode."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure the test option
    options, _ = run_command.parse_options(["--test", "--update-resources"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App will be built with a resource update
        (
            "build",
            "first",
            {
                "test_mode": True,
                "update": False,
                "update_requirements": False,
                "update_resources": True,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Run the first app
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": True, "passthrough": []},
        ),
    ]


def test_test_mode_update_existing_app(run_command, first_app):
    """An existing app can be updated and started in test mode."""
    # Add a single app
    run_command.apps = {
        "first": first_app,
    }

    # Configure the test option
    options, _ = run_command.parse_options(["-u", "--test"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App will be built; update is explicit
        (
            "build",
            "first",
            {
                "test_mode": True,
                "update": True,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Run the first app
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": True, "passthrough": []},
        ),
    ]


def test_test_mode_non_existent(run_command, first_app_config):
    """Requesting a test of a non-existent app causes a create."""
    # Add a single app, using the 'config only' fixture
    run_command.apps = {
        "first": first_app_config,
    }

    # Configure a test option
    options, _ = run_command.parse_options(["--test"])

    # Run the run command
    run_command(**options)

    # The right sequence of things will be done
    assert run_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App will be built in test mode, (which will transitively create)
        (
            "build",
            "first",
            {
                "test_mode": True,
                "update": False,
                "update_requirements": False,
                "update_resources": False,
                "update_support": False,
                "update_stub": False,
                "no_update": False,
            },
        ),
        # App template is verified
        ("verify-app-template", "first"),
        # App tools are verified
        ("verify-app-tools", "first"),
        # Then, it will be started
        (
            "run",
            "first",
            {"build_state": "first", "test_mode": True, "passthrough": []},
        ),
    ]



import pytest

from briefcase.console import Console, Log
from briefcase.platforms.web.static import StaticWebBuildCommand

from ....utils import create_file


@pytest.fixture
def build_command(tmp_path):
    return StaticWebBuildCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )


def test_trim_file(build_command, tmp_path):
    """A file can be trimmed at a sentinel."""
    filename = tmp_path / "dummy.txt"
    content = [
        "This is before the sentinel.",
        "This is also before the sentinel.",
        " ** This is the sentinel ** ",
        "This is after the sentinel.",
        "This is also after the sentinel.",
    ]

    create_file(filename, "\n".join(content))

    # Trim the file at the sentinel
    build_command._trim_file(filename, sentinel=" ** This is the sentinel ** ")

    # The file contains everything up to and including the sentinel.
    with filename.open(encoding="utf-8") as f:
        assert f.read() == "\n".join(content[:3]) + "\n"


def test_trim_no_sentinel(build_command, tmp_path):
    """A file that doesn't contain the sentinel is returned as-is."""
    filename = tmp_path / "dummy.txt"
    content = [
        "This is before the sentinel.",
        "This is also before the sentinel.",
        "NO SENTINEL HERE",
        "This is after the sentinel.",
        "This is also after the sentinel.",
    ]

    create_file(filename, "\n".join(content))

    # Trim the file at a sentinel
    build_command._trim_file(filename, sentinel=" ** This is the sentinel ** ")

    # The file is unmodified.
    with filename.open(encoding="utf-8") as f:
        assert f.read() == "\n".join(content)


def test_trim_file_multiple_sentinels(build_command, tmp_path):
    """A file with multiple sentinels is trimmed at the first one."""
    filename = tmp_path / "dummy.txt"
    content = [
        "This is before the sentinel.",
        "This is also before the sentinel.",
        " ** This is the sentinel ** ",
        "This is after the first sentinel.",
        "This is also after the first sentinel.",
        " ** This is the sentinel ** ",
        "This is after the second sentinel.",
        "This is also after the second sentinel.",
    ]

    create_file(filename, "\n".join(content))

    # Trim the file at the sentinel
    build_command._trim_file(filename, sentinel=" ** This is the sentinel ** ")

    # The file contains everything up to and including the sentinel.
    with filename.open(encoding="utf-8") as f:
        assert f.read() == "\n".join(content[:3]) + "\n"


def test_trim_sentinel_last_line(build_command, tmp_path):
    """A file with the sentinel as the last full line isn't a problem."""
    filename = tmp_path / "dummy.txt"
    content = [
        "This is before the sentinel.",
        "This is also before the sentinel.",
        " ** This is the sentinel ** ",
    ]

    create_file(filename, "\n".join(content) + "\n")

    # Trim the file at a sentinel
    build_command._trim_file(filename, sentinel=" ** This is the sentinel ** ")

    # The file is unmodified.
    with filename.open(encoding="utf-8") as f:
        assert f.read() == "\n".join(content) + "\n"


def test_trim_sentinel_EOF(build_command, tmp_path):
    """A file with the sentinel at EOF isn't a problem."""
    filename = tmp_path / "dummy.txt"
    content = [
        "This is before the sentinel.",
        "This is also before the sentinel.",
        " ** This is the sentinel ** ",
    ]

    create_file(filename, "\n".join(content))

    # Trim the file at a sentinel
    build_command._trim_file(filename, sentinel=" ** This is the sentinel ** ")

    # The file is unmodified.
    with filename.open(encoding="utf-8") as f:
        assert f.read() == "\n".join(content)



import os
import platform
import shutil
import sys
from unittest.mock import MagicMock

import pytest

from briefcase.config import AppConfig
from briefcase.console import Log
from briefcase.integrations.base import ToolCache
from briefcase.integrations.file import File
from briefcase.integrations.subprocess import Subprocess
from tests.utils import DummyConsole


@pytest.fixture
def mock_tools(tmp_path) -> ToolCache:
    mock_tools = ToolCache(
        logger=Log(),
        console=DummyConsole(),
        base_path=tmp_path / "tools",
        home_path=tmp_path / "home",
    )

    # Mock stdlib tools
    mock_tools.os = MagicMock(spec_set=os)
    mock_tools.platform = MagicMock(spec_set=platform)
    mock_tools.shutil = MagicMock(spec_set=shutil)
    mock_tools.sys = MagicMock(spec_set=sys)

    # Mock an empty environment
    mock_tools.os.environ = {}

    # Create base directories
    mock_tools.base_path.mkdir(parents=True)
    mock_tools.home_path.mkdir(parents=True)

    # Make File and Subprocess always available
    File.verify(tools=mock_tools)
    Subprocess.verify(tools=mock_tools)

    return mock_tools


@pytest.fixture
def first_app_config():
    return AppConfig(
        app_name="first-app",
        project_name="First Project",
        formal_name="First App",
        author="Megacorp",
        bundle="com.example",
        version="0.0.1",
        description="The first simple app",
        sources=["src/first_app"],
        license={"file": "LICENSE"},
    )



import pytest

from ....utils import create_file, create_plist_file


@pytest.fixture
def first_app_generated(first_app_config, tmp_path):
    # Create the briefcase.toml file
    create_file(
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "ios"
        / "xcode"
        / "briefcase.toml",
        """
[paths]
app_packages_path="app_packages"
support_path="support"
info_plist_path="Info.plist"
""",
    )

    create_plist_file(
        tmp_path / "base_path/build/first-app/ios/xcode/Info.plist",
        {
            "MainModule": "first_app",
        },
    )
    return first_app_config



import pytest


@pytest.mark.parametrize(
    "app_name, candidate",
    [
        ("helloworld", "helloworld"),
        ("HelloWorld", "HelloWorld"),
        ("hello-world", "hello_world"),
        ("hello_world", "hello_world"),
    ],
)
def test_make_module_name(new_command, app_name, candidate):
    """An app name can be converted into a valid class name."""
    module_name = new_command.make_module_name(app_name)
    assert module_name == candidate



import subprocess
from pathlib import Path

from briefcase.commands import BuildCommand, OpenCommand, PublishCommand, UpdateCommand
from briefcase.config import BaseConfig
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.visualstudio import VisualStudio
from briefcase.platforms.windows import (
    WindowsCreateCommand,
    WindowsMixin,
    WindowsPackageCommand,
    WindowsRunCommand,
)


class WindowsVisualStudioMixin(WindowsMixin):
    output_format = "VisualStudio"
    packaging_root = Path("x64/Release")

    def project_path(self, app):
        return self.bundle_path(app) / f"{app.formal_name}.sln"


class WindowsVisualStudioCreateCommand(WindowsVisualStudioMixin, WindowsCreateCommand):
    description = "Create and populate a Visual Studio project."


class WindowsVisualStudioUpdateCommand(WindowsVisualStudioCreateCommand, UpdateCommand):
    description = "Update an existing Visual Studio project."


class WindowsVisualStudioOpenCommand(WindowsVisualStudioMixin, OpenCommand):
    description = "Open an existing Visual Studio project."


class WindowsVisualStudioBuildCommand(WindowsVisualStudioMixin, BuildCommand):
    description = "Build a Visual Studio project."

    def verify_tools(self):
        super().verify_tools()
        VisualStudio.verify(tools=self.tools)

    def build_app(self, app: BaseConfig, **kwargs):
        """Build the Visual Studio project.

        :param app: The config object for the app
        """
        self.logger.info("Building VisualStudio project...", prefix=app.app_name)

        with self.input.wait_bar("Building solution..."):
            try:
                self.tools.subprocess.run(
                    [
                        self.tools.visualstudio.msbuild_path,
                        f"{app.formal_name}.sln",
                        "-target:restore",
                        "-property:RestorePackagesConfig=true",
                        f"-target:{app.formal_name}",
                        "-property:Configuration=Release",
                        (
                            "-verbosity:detailed"
                            if self.logger.is_deep_debug
                            else "-verbosity:normal"
                        ),
                    ],
                    check=True,
                    cwd=self.bundle_path(app),
                )
            except subprocess.CalledProcessError as e:
                raise BriefcaseCommandError(
                    f"Unable to build solution for {app.app_name}."
                ) from e


class WindowsVisualStudioRunCommand(WindowsVisualStudioMixin, WindowsRunCommand):
    description = "Run a Visual Studio project."


class WindowsVisualStudioPackageCommand(
    WindowsVisualStudioMixin,
    WindowsPackageCommand,
):
    description = "Package a Visual Studio project."


class WindowsVisualStudioPublishCommand(WindowsVisualStudioMixin, PublishCommand):
    description = "Publish a Visual Studio project."


# Declare the briefcase command bindings
create = WindowsVisualStudioCreateCommand
update = WindowsVisualStudioUpdateCommand
open = WindowsVisualStudioOpenCommand
build = WindowsVisualStudioBuildCommand
run = WindowsVisualStudioRunCommand
package = WindowsVisualStudioPackageCommand
publish = WindowsVisualStudioPublishCommand



import pytest

from ...utils import create_file


@pytest.mark.parametrize(
    "unbuilt, built",
    [
        (True, True),
        (True, False),
        (False, True),
        (False, False),
    ],
)
def test_cleanup_stubs(create_command, unbuilt, built, myapp, tmp_path):
    """If an unbuilt stub already exists, it is cleaned up."""

    # Mock a existing stubs
    if unbuilt:
        create_file(
            tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin",
            "Unbuilt stub",
        )
    if built:
        create_file(
            tmp_path / "base_path/build/my-app/tester/dummy/my-app.bin",
            "Built stub",
        )

    # Re-install the support package
    create_command.cleanup_stub_binary(myapp)

    # No matter the starting state, the stubs don't exist after cleanup.
    assert not (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()
    assert not (tmp_path / "base_path/build/my-app/tester/dummy/my-app.bin").exists()



from briefcase.console import Console


def test_default_constructor():
    """A console is enabled by default."""
    console = Console()
    assert console.enabled


def test_constructor_with_enabled_false():
    """A console can be constructed in a disabled state."""
    console = Console(enabled=False)
    assert not console.enabled


def test_enable(disabled_console):
    """A disabled console can be enabled."""
    assert not disabled_console.enabled

    disabled_console.enabled = True

    assert disabled_console.enabled


def test_disable():
    """A disabled console can be enabled."""
    console = Console()
    assert console.enabled

    console.enabled = False
    assert not console.enabled


def test_is_interactive_non_interactive(non_interactive_console):
    """Console is non-interactive when stdout has no tty."""
    assert non_interactive_console.is_interactive is False


def test_is_interactive_always_interactive(console):
    """Console is interactive when stdout has a tty."""
    assert console.is_interactive is True



from __future__ import annotations

import io
import os
import plistlib
import tarfile
import zipfile
from email.message import EmailMessage
from pathlib import Path

import requests
from requests.adapters import HTTPAdapter
from rich.markup import escape
from urllib3.util.retry import Retry

from briefcase.console import Console, InputDisabled


class PartialMatchString(str):
    "A string-like class that has equality on a partial match"

    def __eq__(self, other):
        return self in other


class NoMatchString(str):
    "A string-like class that has equality when there is no match"

    def __eq__(self, other):
        return self not in other


class DummyConsole(Console):
    def __init__(self, *values, enabled=True):
        super().__init__(enabled=enabled)
        self.prompts = []
        self.values = list(values)

    def __call__(self, prompt, *args, **kwargs):
        if not self.enabled:
            raise InputDisabled()
        self.prompts.append(prompt)
        return self.values.pop(0)


def default_rich_prompt(prompt: str) -> str:
    """Formats a prompt as what is actually passed to Rich."""
    return f"[bold]{escape(prompt)}[/bold]"


def create_file(filepath, content, mode="w", chmod=None):
    """A test utility to create a file with known content.

    Ensures that the directory for the file exists, and writes a file with
    specific content.

    :param filepath: The path for the file to create
    :param content: A string containing the content to write.
    :param mode: The mode to open the file. This is `w` by default;
        use `wb` and provide content as a bitstring if you need to
        write a binary file.
    :param chmod: file permissions to apply
    :returns: The path to the file that was created.
    """
    filepath.parent.mkdir(parents=True, exist_ok=True)
    with filepath.open(mode, **({} if "b" in mode else {"encoding": "utf-8"})) as f:
        f.write(content)

    if chmod:
        os.chmod(filepath, chmod)

    return filepath


def create_plist_file(plistpath, content):
    """A test utility to create a plist file with known content.

    Ensures that the directory for the file exists, and writes an XML plist with
    specific content.

    :param plistpath: The path for the plist file to create.
    :param content: A dictionary of content that plistlib can use to create the plist
        file.
    :returns: The path to the file that was created.
    """
    plistpath.parent.mkdir(parents=True, exist_ok=True)
    with plistpath.open("wb") as f:
        plistlib.dump(content, f)

    return plistpath


def create_zip_file(zippath, content):
    """A test utility to create a .zip file with known content.

    Ensures that the directory for the file exists, and writes a file with specific
    content.

    :param zippath: The path for the ZIP file to create
    :param content: A list of pairs; each pair is (path, data) describing an item to be
        added to the zip file.
    :returns: The path to the file that was created.
    """
    zippath.parent.mkdir(parents=True, exist_ok=True)
    with zipfile.ZipFile(zippath, "w") as f:
        for path, data in content:
            f.writestr(path, data=data)

    return zippath


def create_tgz_file(tgzpath, content):
    """A test utility to create a .tar.gz file with known content.

    Ensures that the directory for the file exists, and writes a file with specific
    content.

    :param tgzpath: The path for the gzipped tarfile file to create
    :param content: A list of pairs; each pair is (path, data) describing an item to be
        added to the zip file.
    :returns: The path to the file that was created.
    """
    tgzpath.parent.mkdir(parents=True, exist_ok=True)
    with tarfile.open(tgzpath, "w:gz") as f:
        for path, data in content:
            tarinfo = tarfile.TarInfo(path)
            payload = data.encode("utf-8")
            tarinfo.size = len(payload)
            f.addfile(tarinfo, io.BytesIO(payload))

    return tgzpath


def mock_file_download(filename, content, mode="w", role=None):
    """Create a side effect function that mocks the download of a zip file.

    :param filename: The file name (*not* the path - just the file name) to
        create as a side effect
    :param content: A string containing the content to write.
    :param mode: The mode to open the file. This is `w` by default;
        use `wb` and provide content as a bitstring if you need to
        write a binary file.
    :param role: The role played by the content being downloaded
    :returns: a function that can act as a mock side effect for `file.download()`
    """

    def _download_file(url, download_path, role):
        return create_file(download_path / filename, content, mode=mode)

    return _download_file


def mock_zip_download(filename, content, role=None):
    """Create a side effect function that mocks the download of a zip file.

    :param filename: The file name (*not* the path - just the file name) to
        create as a side effect
    :param content: A string containing the content to write.
    :param role: The role played by the content being downloaded
    :returns: a function that can act as a mock side effect for `file.download()`
    """

    def _download_file(url, download_path, role):
        return create_zip_file(download_path / filename, content)

    return _download_file


def mock_tgz_download(filename, content, role=None):
    """Create a side effect function that mocks the download of a .tar.gz file.

    :param content: A string containing the content to write.
    :returns: a function that can act as a mock side effect for `file.download()`
    """

    def _download_file(url, download_path, role):
        return create_tgz_file(download_path / filename, content)

    return _download_file


def distinfo_metadata(
    package: str = "dummy",
    version: str = "1.2.3",
    tag: str = "py3-none-any",
):
    """Generate the content for a distinfo folder.

    :param package: The name of the package.
    :param version: The version number of the package.
    :param tag: The packaging tag for the package.
    """
    content = []

    # INSTALLER
    installer = "pip\n"
    content.append((f"{package}-{version}.dist-info/INSTALLER", installer))

    # METADATA
    metadata = EmailMessage()
    metadata["Metadata-Version"] = "2.1"
    metadata["Name"] = package
    metadata["Version"] = version
    metadata["Summary"] = f"A packaged named {package}."
    metadata["Author-email"] = "Jane Developer <jane@example.com>"
    content.append((f"{package}-{version}.dist-info/METADATA", str(metadata)))

    # WHEEL
    wheel = EmailMessage()
    wheel["Wheel-Version"] = "1.0"
    wheel["Generator"] = "test-case"
    wheel["Root-Is-Purelib"] = "true" if tag == "py3-none-any" else "false"
    wheel["Tag"] = tag
    content.append((f"{package}-{version}.dist-info/WHEEL", str(wheel)))

    # RECORD
    # Create the file, but don't actually populate it.
    record = ""
    content.append((f"{package}-{version}.dist-info/RECORD", record))

    return content


def installed_package_content(
    package="dummy",
    version="1.2.3",
    tag="py3-none-any",
    extra_content=None,
):
    """Generate the content for an installed package.

    :param path: The site-packages folder into which to install the package.
    :param package: The name of the package in the wheel. Defaults to ``dummy``
    :param version: The version number of the package. Defaults to ``1.2.3``
    :param tag: The installation tag for the package. Defaults to a pure python wheel.
    :param extra_content: Optional. A list of tuples of ``(path, content)`` that will be
        added to the wheel.
    """
    return (
        [
            (f"{package}/__init__.py", ""),
            (f"{package}/app.py", "# This is the app"),
        ]
        + (extra_content if extra_content else [])
        + distinfo_metadata(package=package, version=version, tag=tag)
    )


def create_installed_package(
    path,
    package="dummy",
    version="1.2.3",
    tag="py3-none-any",
    extra_content=None,
):
    """Write an installed package into a 'site-packages' folder.

    :param path: The site-packages folder into which to install the package.
    :param package: The name of the package in the wheel. Defaults to ``dummy``
    :param version: The version number of the package. Defaults to ``1.2.3``
    :param tag: The installation tag for the package. Defaults to a pure python wheel.
    :param extra_content: Optional. A list of tuples of ``(path, content)`` or
        ``(path, content, chmod)`` that will be added to the wheel. If ``chmod`` is
        not specified, default filesystem permissions will be used.
    """
    for entry in installed_package_content(
        package=package,
        version=version,
        tag=tag,
        extra_content=extra_content,
    ):
        try:
            filename, content, chmod = entry
        except ValueError:
            filename, content = entry
            chmod = None
        create_file(path / filename, content=content, chmod=chmod)


def create_wheel(
    path,
    package="dummy",
    version="1.2.3",
    tag="py3-none-any",
    extra_content=None,
):
    """Create a sample wheel file.

    :param path: The folder where the wheel should be written.
    :param package: The name of the package in the wheel. Defaults to ``dummy``
    :param version: The version number of the package. Defaults to ``1.2.3``
    :param tag: The installation tag for the package. Defaults to a pure python wheel.
    :param extra_content: Optional. A list of tuples of ``(path, content)`` or
        ``(path, content, chmod)`` that will be added to the wheel. If ``chmod`` is
        not specified, default filesystem permissions will be used.
    """
    wheel_filename = path / f"{package}-{version}-{tag}.whl"

    create_zip_file(
        wheel_filename,
        content=installed_package_content(
            package=package,
            version=version,
            tag=tag,
            extra_content=extra_content,
        ),
    )

    return wheel_filename


def file_content(path: Path) -> str | None:
    """Return the content of a file, or None if the path is a directory."""
    if path.is_dir():
        return None
    with path.open(encoding="utf-8") as f:
        return f.read()


def assert_url_resolvable(url: str):
    """Tests whether a URL is resolvable with retries; raises for failure."""
    with requests.session() as sess:
        adapter = HTTPAdapter(max_retries=Retry(status_forcelist={500, 502, 504}))
        sess.mount("http://", adapter)
        sess.mount("https://", adapter)

        sess.head(url).raise_for_status()



import pytest


@pytest.mark.parametrize(
    "name, bundle, candidate",
    [
        ("Jane Developer", "com.example", "jane@example.com"),
        ("Jane Developer", "com.example.more", "jane@more.example.com"),
        ("Jane", "com.example", "jane@example.com"),
    ],
)
def test_make_author_email(new_command, name, bundle, candidate):
    """An author name and bundle can be converted into a valid email address."""
    email = new_command.make_author_email(name, bundle)
    assert email == candidate
    # Double check - the app name passes the validity check.
    assert new_command.validate_email(email)



from unittest.mock import MagicMock

import pytest

from briefcase.exceptions import UnsupportedHostError


def test_default_options(create_command):
    """The default options are as expected."""
    options, overrides = create_command.parse_options([])

    assert options == {}
    assert overrides == {}

    assert create_command.target_image is None


def test_options(create_command):
    """The extra options can be parsed."""
    options, overrides = create_command.parse_options(
        [
            "--target",
            "somevendor:surprising",
            "--Xdocker-build=--option-one",
            "--Xdocker-build=--option-two",
        ]
    )

    assert options == {}
    assert overrides == {}

    assert create_command.target_image == "somevendor:surprising"
    assert create_command.extra_docker_build_args == ["--option-one", "--option-two"]


@pytest.mark.parametrize("host_os", ["Windows", "WeirdOS"])
def test_unsupported_host_os_with_docker(create_command, host_os, tmp_path):
    """Error raised for an unsupported OS when using Docker."""
    create_command.target_image = "somevendor:surprising"
    create_command.tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match="Linux system projects can only be built on Linux, or on macOS using Docker.",
    ):
        create_command()


@pytest.mark.parametrize("host_os", ["Darwin", "Windows", "WeirdOS"])
def test_unsupported_host_os_without_docker(create_command, host_os, tmp_path):
    """Error raised for an unsupported OS when not using Docker."""
    create_command.target_image = None
    create_command.tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match="Linux system projects can only be built on Linux, or on macOS using Docker.",
    ):
        create_command()


def test_supported_host_os_docker(create_command):
    """If using Docker on a supported host, no error is raised."""
    create_command.target_image = "somevendor:surprising"
    create_command.tools.host_os = "Linux"

    # Verify the host
    create_command.verify_host()


def test_supported_host_os_without_docker(create_command):
    """If not using Docker on a supported host, no error is raised."""
    create_command.target_image = None
    create_command.tools.host_os = "Linux"

    # Verify the host
    create_command.verify_host()


@pytest.mark.parametrize(
    "is_user_mapped, host_os, use_non_root",
    [
        (False, "Darwin", True),
        (True, "Darwin", True),
        (False, "Linux", True),
        (True, "Linux", False),
    ],
)
def test_output_format_template_context(
    create_command,
    first_app_config,
    is_user_mapped,
    host_os,
    use_non_root,
):
    """The template context contains additional deb-specific properties."""
    # Add some properties defined in config finalization
    first_app_config.python_version_tag = "3.X"
    first_app_config.target_image = "somevendor:surprising"
    first_app_config.target_vendor = "somevendor"
    first_app_config.target_codename = "surprising"
    first_app_config.target_vendor_base = "basevendor"
    first_app_config.glibc_version = "2.42"

    # Mock the host OS
    create_command.tools.host_os = host_os

    # Mock a target Docker image
    create_command.target_image = "somearch:surprising"

    # Mock Docker user mapping setting for `use_non_root_user`
    create_command.tools.docker = MagicMock()
    create_command.tools.docker.is_user_mapped = is_user_mapped

    # Add a long description
    first_app_config.long_description = "This is a long\ndescription."

    # Generate the context
    context = create_command.output_format_template_context(first_app_config)

    # Context extras are what we expect
    assert context == {
        "format": "surprising",
        "python_version": "3.X",
        "docker_base_image": "somevendor:surprising",
        "vendor_base": "basevendor",
        "use_non_root_user": use_non_root,
    }


def test_output_format_template_context_no_docker(create_command, first_app_config):
    """If not using Docker, `use_non_root_user` default in template is used."""
    # Mock the host to Linux to avoid flagging any "always use non-root user on macOS" logic.
    create_command.tools.host_os = "Linux"

    # Add some properties defined in config finalization
    first_app_config.python_version_tag = "3.X"
    first_app_config.target_image = "somevendor:surprising"
    first_app_config.target_vendor = "somevendor"
    first_app_config.target_codename = "surprising"
    first_app_config.target_vendor_base = "basevendor"
    first_app_config.glibc_version = "2.42"

    context = create_command.output_format_template_context(first_app_config)

    # Context extras does not contain `use_non_root_user`
    assert context == {
        "format": "surprising",
        "python_version": "3.X",
        "docker_base_image": "somevendor:surprising",
        "vendor_base": "basevendor",
    }
